@@ -17,13 +17,12 @@
  */
 #include "bstapp.h"
 
-#include "../PKG_config.h"
+#include "PKG_config.h"
 
 #include "bstsongshell.h"
 #include "bstwavereposhell.h"
 #include "bstsnetshell.h"
 #include "bstfiledialog.h"
-#include "bststatusbar.h"
 #include "bstgconfig.h"
 #include "bstpreferences.h"
 #include "bstprocbrowser.h"
@@ -31,30 +30,35 @@
 #include "bstrackeditor.h"
 #include "bstmenus.h"
 #include "bstprocedure.h"
-#include "bsttexttools.h"
+#include "bstprojectctrl.h"
+#include "sfi/toyprof-mem.h"
 
 
 /* --- prototypes --- */
-static void	bst_app_class_init		(BstAppClass	 *class);
-static void	bst_app_init			(BstApp		 *app);
-static void	bst_app_destroy			(GtkObject	 *object);
-static gboolean bst_app_handle_delete_event	(GtkWidget	 *widget,
-						 GdkEventAny	 *event);
-static void     bst_app_run_script_proc		(GtkWidget	 *widget,
-						 gulong           callback_action,
-						 gpointer         popup_data);
+static void	bst_app_class_init		(BstAppClass	*class);
+static void	bst_app_init			(BstApp		*app);
+static void	bst_app_destroy			(GtkObject	*object);
+static gboolean bst_app_handle_delete_event	(GtkWidget	*widget,
+						 GdkEventAny	*event);
+static void	bst_app_menu_callback		(GtkWidget	*owner,
+						 gulong		 callback_action,
+						 gpointer	 popup_data);
+static void     bst_app_run_script_proc		(GtkWidget	*widget,
+						 gulong          category_id,
+						 gpointer        popup_data);
 
 
 /* --- menus --- */
 static gchar	   *bst_app_factories_path = "<BstApp>";
-static GtkItemFactoryEntry menubar_entries[] =
+static BstMenuConfigEntry menubar_entries[] =
 {
-#define BST_OP(bst_op) (bst_app_operate), (BST_OP_ ## bst_op)
+#define BST_OP(bst_op) (bst_app_menu_callback), (BST_OP_ ## bst_op)
   { "/_File",				NULL,		NULL, 0,			"<Branch>" },
   { "/File/<<<<<<",			NULL,		NULL, 0,			"<Tearoff>" },
   { "/File/_New",			"<ctrl>N",	BST_OP (PROJECT_NEW),		"<Item>" },
   { "/File/_Open...",			"<ctrl>O",	BST_OP (PROJECT_OPEN),		"<Item>" },
-  { "/File/_Save",			"<ctrl>S",	BST_OP (PROJECT_SAVE),		"<Item>" },
+  { "/File/_Merge...",			"<ctrl>M",	BST_OP (PROJECT_MERGE),		"<Item>" },
+  { "/File/_Save",			NULL,		BST_OP (PROJECT_SAVE),		"<Item>" },
   { "/File/Save _As...",		NULL,		BST_OP (PROJECT_SAVE_AS),	"<Item>" },
   { "/File/-----",			NULL,		NULL, 0,			"<Separator>" },
   { "/File/_Dialogs",			NULL,		NULL, 0,			"<Branch>" },
@@ -77,7 +81,6 @@ static GtkItemFactoryEntry menubar_entries[] =
   { "/Project/Rack Editor",		NULL,		BST_OP (PROJECT_RACK_EDITOR),	"<Item>" },
   { "/Project/-----",			NULL,		NULL, 0,			"<Separator>" },
   { "/Project/Rebuild",			NULL,		BST_OP (REBUILD),		"<Item>" },
-  { "/Project/Refresh",			NULL,		BST_OP (REFRESH),		"<Item>" },
   { "/_Edit",				NULL,		NULL, 0,			"<Branch>" },
   { "/Edit/<<<<<<",			NULL,		NULL, 0,			"<Tearoff>" },
   { "/Edit/_Undo",			"<ctrl>Z",	BST_OP (UNDO_LAST),		"<Item>" },
@@ -98,7 +101,7 @@ static GtkItemFactoryEntry menubar_entries[] =
   // { "/SNet/<<<<<<",			NULL,		NULL, 0,			"<Tearoff>" },
   // { "/SNet/_Test",			"",		BST_OP (NONE),			"<Item>" },
 };
-static GtkItemFactoryEntry menubar_help_entries[] = {
+static BstMenuConfigEntry menubar_help_entries[] = {
   { "/_Help",				NULL,		NULL, 0,			"<LastBranch>" },
   { "/Help/<<<<<<",			NULL,		NULL, 0,			"<Tearoff>" },
   { "/Help/_Release Notes...",		NULL,		BST_OP (HELP_RELEASE_NOTES),	"<Item>" },
@@ -136,7 +139,7 @@ bst_app_get_type (void)
 	(GtkClassInitFunc) NULL,
       };
 
-      app_type = gtk_type_unique (BST_TYPE_DIALOG, &app_info);
+      app_type = gtk_type_unique (GXK_TYPE_DIALOG, &app_info);
     }
 
   return app_type;
@@ -176,15 +179,13 @@ bst_app_init (BstApp *app)
   GtkWidget *widget = GTK_WIDGET (app);
   GtkWindow *window = GTK_WINDOW (app);
   GtkItemFactory *factory;
-  GtkItemFactoryEntry *centries;
-  BseCategory *cats;
-  GSList *slist;
-  guint n_cats;
+  BseCategorySeq *cseq;
+  BstMenuConfig *m1, *m2;
   
   g_object_set (app,
 		"allow_shrink", TRUE,
 		"allow_grow", TRUE,
-		"flags", BST_DIALOG_STATUS_SHELL,
+		"flags", GXK_DIALOG_STATUS_SHELL,
 		NULL);
   bst_app_register (app);
   if (0)
@@ -195,7 +196,7 @@ bst_app_init (BstApp *app)
   bst_app_register (app);
   app->main_vbox = g_object_connect (gtk_widget_new (GTK_TYPE_VBOX,
 						     "visible", TRUE,
-						     "parent", BST_DIALOG (app)->vbox,
+						     "parent", GXK_DIALOG (app)->vbox,
 						     NULL),
 				     "swapped_signal::destroy", g_nullify_pointer, &app->main_vbox,
 				     NULL);
@@ -215,28 +216,29 @@ bst_app_init (BstApp *app)
 			    bst_app_factories_path,
 			    factory,
 			    (GtkDestroyNotify) gtk_object_unref);
+  /* standard entries */
+  m1 = bst_menu_config_from_entries (G_N_ELEMENTS (menubar_entries), menubar_entries);
+  /* script entries */
+  cseq = bse_categories_match_typed ("/Scripts/*", "BseProcedure");
+  m2 = bst_menu_config_from_cats (cseq, bst_app_run_script_proc, FALSE);
+  bst_menu_config_sort (m2);
+  /* merge and sort up */
+  m1 = bst_menu_config_merge (m1, m2);
+  /* add help entries */
+  m2 = bst_menu_config_from_entries (G_N_ELEMENTS (menubar_help_entries), menubar_help_entries);
+  m1 = bst_menu_config_merge (m1, m2);
+  /* and create menu items */
+  bst_menu_config_create_items (m1, factory, GTK_WIDGET (app));
+  bst_menu_config_free (m1);
 
-  /* setup standard entries
-   */
-  gtk_item_factory_create_items (factory, G_N_ELEMENTS (menubar_entries), menubar_entries, app);
-
-  /* add scripts to the menu bar
-   */
-  cats = bse_categories_match_typed ("/Scripts/*", BSE_TYPE_PROCEDURE, &n_cats);
-  centries = bst_menu_entries_from_cats (n_cats, cats, bst_app_run_script_proc, FALSE);
-  slist = bst_menu_entries_slist (n_cats, centries);
-  slist = bst_menu_entries_sort (slist);
-  bst_menu_entries_create (factory, slist, GTK_WIDGET (app));
-  g_slist_free (slist);
-  g_free (centries);
-  g_free (cats);
-
-  /* setup help entries
-   */
-  gtk_item_factory_create_items (factory, G_N_ELEMENTS (menubar_help_entries), menubar_help_entries, app);
-
-  /* setup the main notebook
-   */
+  /* setup playback controls */
+  app->pcontrols = g_object_new (BST_TYPE_PROJECT_CTRL, NULL);
+  gtk_container_add_with_properties (GTK_CONTAINER (app->main_vbox),
+				     app->pcontrols,
+				     "expand", FALSE,
+				     "position", 1,
+				     NULL);
+  /* setup the main notebook */
   app->notebook = g_object_new (GTK_TYPE_NOTEBOOK,
 				"visible", TRUE,
 				"parent", app->main_vbox,
@@ -261,11 +263,13 @@ bst_app_destroy (GtkObject *object)
 
   if (app->project)
     {
-      bsw_server_halt_project (BSW_SERVER, app->project);
-      g_object_disconnect (bse_object_from_id (app->project),
+      if (app->pcontrols)
+	bst_project_ctrl_set_project (BST_PROJECT_CTRL (app->pcontrols), 0);
+      bse_project_deactivate (app->project);
+      bse_proxy_disconnect (app->project,
 			   "any_signal", bst_app_reload_supers, app,
 			   NULL);
-      bsw_item_unuse (app->project);
+      bse_item_unuse (app->project);
       app->project = 0;
     }
 
@@ -276,18 +280,18 @@ bst_app_destroy (GtkObject *object)
   if (!bst_app_class->apps && bst_app_class->seen_apps)
     {
       bst_app_class->seen_apps = FALSE;
-      gtk_main_quit ();
+      BST_MAIN_LOOP_QUIT ();
     }
 }
 
 BstApp*
-bst_app_new (BswProxy project)
+bst_app_new (SfiProxy project)
 {
   GdkGeometry geometry;
   GtkWidget *widget;
   BstApp *app;
 
-  g_return_val_if_fail (BSW_IS_PROJECT (project), NULL);
+  g_return_val_if_fail (BSE_IS_PROJECT (project), NULL);
 
   widget = gtk_widget_new (BST_TYPE_APP,
 			   "default_width", 640,
@@ -300,13 +304,15 @@ bst_app_new (BswProxy project)
   gtk_window_set_geometry_hints (GTK_WINDOW (widget), NULL, &geometry, GDK_HINT_MIN_SIZE);
 
   app->project = project;
-  bsw_item_use (app->project);
-  g_object_connect (bse_object_from_id (app->project),
-		    "swapped_signal::item-added", bst_app_reload_supers, app,
-		    "swapped_signal::item-removed", bst_app_reload_supers, app,
-		    NULL);
-  bst_dialog_sync_title_to_proxy (BST_DIALOG (app), app->project, "%s");
-
+  bse_item_use (app->project);
+  bse_proxy_connect (app->project,
+		     "swapped_signal::item-added", bst_app_reload_supers, app,
+		     "swapped_signal::item-removed", bst_app_reload_supers, app,
+		     NULL);
+  bst_window_sync_title_to_proxy (GXK_DIALOG (app), app->project, "%s");
+  if (app->pcontrols)
+    bst_project_ctrl_set_project (BST_PROJECT_CTRL (app->pcontrols), app->project);
+  
   bst_app_reload_supers (app);
 
   /* update menu entries
@@ -317,11 +323,11 @@ bst_app_new (BswProxy project)
 }
 
 BstApp*
-bst_app_find (BswProxy project)
+bst_app_find (SfiProxy project)
 {
   GSList *slist;
   
-  g_return_val_if_fail (BSW_IS_PROJECT (project), NULL);
+  g_return_val_if_fail (BSE_IS_PROJECT (project), NULL);
 
   for (slist = bst_app_class->apps; slist; slist = slist->next)
     {
@@ -358,24 +364,24 @@ bst_app_menu_factory (BstApp *app)
 
 static GtkWidget*
 bst_app_create_super_shell (BstApp  *app,
-			    BswProxy super)
+			    SfiProxy super)
 {
   GtkWidget *shell = NULL;
 
-  if (BSW_IS_SONG (super))
+  if (BSE_IS_SONG (super))
     shell = gtk_widget_new (BST_TYPE_SONG_SHELL,
 			    "visible", TRUE,
 			    NULL);
-  else if (BSW_IS_SNET (super))
+  else if (BSE_IS_SNET (super))
     shell = gtk_widget_new (BST_TYPE_SNET_SHELL,
 			    "visible", TRUE,
 			    NULL);
-  else if (BSW_IS_WAVE_REPO (super))
+  else if (BSE_IS_WAVE_REPO (super))
     shell = gtk_widget_new (BST_TYPE_WAVE_REPO_SHELL,
 			    "visible", TRUE,
 			    NULL);
   else
-    g_warning ("unknown super type `%s'", bsw_item_get_type_name (super));
+    g_warning ("unknown super type `%s'", bse_item_get_type_name (super));
 
   if (shell)
     {
@@ -391,10 +397,12 @@ bst_app_create_super_shell (BstApp  *app,
 void
 bst_app_reload_supers (BstApp *app)
 {
-  GtkWidget *old_page, *old_focus;
+  GtkWidget *old_page, *old_focus, *song_page = NULL, *synth_page = NULL;
   GSList *page_list = NULL;
   GSList *slist;
-
+  BseProxySeq *pseq;
+  guint i;
+  
   g_return_if_fail (BST_IS_APP (app));
 
   old_focus = GTK_WINDOW (app)->focus_widget;
@@ -408,38 +416,44 @@ bst_app_reload_supers (BstApp *app)
       gtk_container_remove (GTK_CONTAINER (app->notebook), page_list->data);
     }
 
-  for (slist = BSE_PROJECT (bse_object_from_id (app->project))->supers; slist; slist = slist->next)
+  pseq = bse_project_get_supers (app->project);
+  for (i = 0; i < pseq->n_proxies; i++)
     {
       GtkWidget *label, *page = NULL;
       GSList *node;
 
       for (node = page_list; node; node = node->next)
-	if (BST_SUPER_SHELL (node->data)->super == BSE_OBJECT_ID (slist->data))
+	if (BST_SUPER_SHELL (node->data)->super == pseq->proxies[i])
 	  {
 	    page = node->data;
 	    page_list = g_slist_remove (page_list, page);
 	    break;
 	  }
       if (!page)
-	page = bst_app_create_super_shell (app, BSE_OBJECT_ID (slist->data));
+	page = bst_app_create_super_shell (app, pseq->proxies[i]);
       if (page)
 	{
-	  label = gtk_widget_new (GTK_TYPE_LABEL,
-				  "visible", TRUE,
-				  "width_request", BST_TAB_WIDTH ? BST_TAB_WIDTH : -1,
-				  NULL);
-	  gtk_notebook_append_page (app->notebook,
-				    page,
-				    label);
+	  if (!song_page && BSE_IS_SONG (pseq->proxies[i]))
+	    song_page = page;
+	  else if (!synth_page && BSE_IS_SNET (pseq->proxies[i]))
+	    synth_page = page;
+	  label = g_object_new (GTK_TYPE_LABEL,
+				"visible", TRUE,
+				"width_request", BST_TAB_WIDTH ? BST_TAB_WIDTH : -1,
+				NULL);
+	  gtk_notebook_append_page (app->notebook, page, label);
 	  gtk_notebook_set_tab_label_packing (app->notebook, page, FALSE, TRUE, GTK_PACK_START);
-	  bst_super_shell_update_parent (BST_SUPER_SHELL (page));
+	  bst_super_shell_update_label (BST_SUPER_SHELL (page));
 	  gtk_widget_unref (page);
 	}
     }
   if (old_page && old_page->parent == GTK_WIDGET (app->notebook))
-    gtk_notebook_set_current_page (app->notebook,
-				   gtk_notebook_page_num (app->notebook,
-							  old_page));
+    gxk_notebook_set_current_page_widget (app->notebook, old_page);
+  else if (song_page)
+    gxk_notebook_set_current_page_widget (app->notebook, song_page);
+  else if (synth_page)
+    gxk_notebook_set_current_page_widget (app->notebook, synth_page);
+
   if (old_focus)
     {
       if (old_page && gtk_widget_is_ancestor (old_focus, old_page) &&
@@ -484,9 +498,21 @@ bst_app_update_can_operate (BstApp *app)
 
   shell = bst_app_get_current_shell (app);
   widget = gtk_item_factory_get_item (bst_app_menu_factory (app), "/Song");
-  gtk_widget_set_sensitive (widget, BST_IS_SONG_SHELL (shell));
+  if (widget)
+    gtk_widget_set_sensitive (widget, BST_IS_SONG_SHELL (shell));
   widget = gtk_item_factory_get_item (bst_app_menu_factory (app), "/Waves");
-  gtk_widget_set_sensitive (widget, BST_IS_WAVE_REPO_SHELL (shell));
+  if (widget)
+    gtk_widget_set_sensitive (widget, BST_IS_WAVE_REPO_SHELL (shell));
+
+  widget = gtk_item_factory_get_item (bst_app_menu_factory (app), "/File/Save");	// FIXME: hack
+  if (widget)
+    gtk_widget_set_sensitive (widget, BST_IS_SONG_SHELL (shell));
+  widget = gtk_item_factory_get_item (bst_app_menu_factory (app), "/Edit/Undo");	// FIXME: hack
+  if (widget)
+    gtk_widget_set_sensitive (widget, BST_IS_SONG_SHELL (shell));
+  widget = gtk_item_factory_get_item (bst_app_menu_factory (app), "/Edit/Redo");	// FIXME: hack
+  if (widget)
+    gtk_widget_set_sensitive (widget, BST_IS_SONG_SHELL (shell));
 
   for (i = BST_OP_NONE; i < BST_OP_LAST; i++)
     {
@@ -510,37 +536,46 @@ foreach_super_shell_operate (BstSuperShell *shell,
 static void
 rebuild_super_shell (BstSuperShell *super_shell)
 {
-  BswProxy proxy;
+  SfiProxy proxy;
 
   g_return_if_fail (BST_IS_SUPER_SHELL (super_shell));
 
   proxy = super_shell->super;
-  bsw_item_use (proxy);
+  bse_item_use (proxy);
   bst_super_shell_set_super (super_shell, 0);
   bst_super_shell_set_super (super_shell, proxy);
-  bsw_item_unuse (proxy);
+  bse_item_unuse (proxy);
 }
 
 static void
 bst_app_run_script_proc (GtkWidget *widget,
-			 gulong     callback_action,
+			 gulong     category_id,
 			 gpointer   popup_data)
 {
   BstApp *self = BST_APP (widget);
-  GType proc_type = callback_action;
+  BseCategory *cat = bse_category_from_id (category_id);
 
-  bst_procedure_exec_auto (proc_type,
-			   "project", BSE_TYPE_PROJECT, bse_object_from_id (self->project),
+  bst_procedure_exec_auto (cat->type,
+			   "project", SFI_TYPE_PROXY, self->project,
 			   NULL);
 }
 
+static void
+bst_app_menu_callback (GtkWidget *owner,
+		       gulong     callback_action,
+		       gpointer   popup_data)
+{
+  bst_app_operate (BST_APP (owner), callback_action);
+}
+
 void
 bst_app_operate (BstApp *app,
 		 BstOps	 op)
 {
   static GtkWidget *bst_help_dialogs[BST_OP_HELP_LAST - BST_OP_HELP_FIRST + 1] = { NULL, };
-  static GtkWidget *bst_dialog_open = NULL;
-  static GtkWidget *bst_dialog_save = NULL;
+  static GtkWidget *gxk_dialog_open = NULL;
+  static GtkWidget *gxk_dialog_merge = NULL;
+  static GtkWidget *gxk_dialog_save = NULL;
   static GtkWidget *bst_preferences = NULL;
   static GtkWidget *bst_proc_browser = NULL;
   GtkWidget *widget, *shell;
@@ -554,47 +589,57 @@ bst_app_operate (BstApp *app,
 
   gtk_widget_ref (widget);
 
-  bst_status_window_push (widget);
+  gxk_status_window_push (widget);
 
   switch (op)
     {
       BstSuperShell *super_shell;
-      BswProxy proxy;
+      SfiProxy proxy;
       GtkWidget *any;
     case BST_OP_PROJECT_NEW:
       if (1)
 	{
-	  BswProxy project = bsw_server_use_new_project (BSW_SERVER, "Untitled.bse");
+	  SfiProxy project = bse_server_use_new_project (BSE_SERVER, "Untitled.bse");
 	  BstApp *new_app;
 
-	  bsw_project_ensure_wave_repo (project);
+	  bse_project_ensure_wave_repo (project);
 	  new_app = bst_app_new (project);
-	  bsw_item_unuse (project);
+	  bse_item_unuse (project);
 
-	  gtk_idle_show_widget (GTK_WIDGET (new_app));
+	  gxk_idle_show_widget (GTK_WIDGET (new_app));
 	}
       break;
     case BST_OP_PROJECT_OPEN:
-      if (!bst_dialog_open)
+      if (!gxk_dialog_open)
 	{
-	  bst_dialog_open = bst_file_dialog_new_open (app);
-	  g_object_connect (bst_dialog_open,
-			    "signal::destroy", gtk_widget_destroyed, &bst_dialog_open,
+	  gxk_dialog_open = bst_file_dialog_new_open (app, 0);
+	  g_object_connect (gxk_dialog_open,
+			    "signal::destroy", gtk_widget_destroyed, &gxk_dialog_open,
 			    NULL);
 	}
-      gtk_widget_showraise (bst_dialog_open);
+      gxk_widget_showraise (gxk_dialog_open);
+      break;
+    case BST_OP_PROJECT_MERGE:
+      if (!gxk_dialog_merge)
+	{
+	  gxk_dialog_merge = bst_file_dialog_new_open (app, app->project);
+	  g_object_connect (gxk_dialog_merge,
+			    "signal::destroy", gtk_widget_destroyed, &gxk_dialog_merge,
+			    NULL);
+	}
+      gxk_widget_showraise (gxk_dialog_merge);
       break;
     case BST_OP_PROJECT_SAVE_AS:
-      if (bst_dialog_save)
-	gtk_widget_destroy (bst_dialog_save);
-      bst_dialog_save = bst_file_dialog_new_save (app);
-      g_object_connect (bst_dialog_save,
-			"signal::destroy", gtk_widget_destroyed, &bst_dialog_save,
+      if (gxk_dialog_save)
+	gtk_widget_destroy (gxk_dialog_save);
+      gxk_dialog_save = bst_file_dialog_new_save (app);
+      g_object_connect (gxk_dialog_save,
+			"signal::destroy", gtk_widget_destroyed, &gxk_dialog_save,
 			NULL);
-      gtk_widget_showraise (bst_dialog_save);
+      gxk_widget_showraise (gxk_dialog_save);
       break;
     case BST_OP_PROJECT_CLOSE:
-      gtk_toplevel_delete (widget);
+      gxk_toplevel_delete (widget);
       break;
     case BST_OP_EXIT:
       if (bst_app_class)
@@ -602,39 +647,27 @@ bst_app_operate (BstApp *app,
 	  GSList *slist, *free_slist = g_slist_copy (bst_app_class->apps);
 
 	  for (slist = free_slist; slist; slist = slist->next)
-	    gtk_toplevel_delete (slist->data);
+	    gxk_toplevel_delete (slist->data);
 	  g_slist_free (free_slist);
 	}
       break;
     case BST_OP_PROJECT_NEW_SONG:
-      proxy = bsw_project_create_song (app->project, NULL);
+      proxy = bse_project_create_song (app->project, NULL);
       super_shell = bst_super_shell_from_super (proxy);
       break;
     case BST_OP_PROJECT_NEW_SNET:
-      proxy = bsw_project_create_snet (app->project, NULL);
+      proxy = bse_project_create_snet (app->project, NULL);
       super_shell = bst_super_shell_from_super (proxy);
       break;
     case BST_OP_PROJECT_NEW_MIDI_SYNTH:
-      proxy = bsw_project_create_midi_synth (app->project, NULL);
+      proxy = bse_project_create_midi_synth (app->project, NULL);
       super_shell = bst_super_shell_from_super (proxy);
       break;
     case BST_OP_PROJECT_PLAY:
-      {
-	gchar *starting;
-	BswErrorType error;
-
-	if (bsw_project_is_playing (app->project))
-	  starting = "Restarting Playback";
-	else
-	  starting = "Starting Playback";
-
-	error = bsw_server_run_project (BSW_SERVER, app->project);
-	bst_status_eprintf (error, starting);
-      }
+      bst_project_ctrl_play (BST_PROJECT_CTRL (app->pcontrols));
       break;
     case BST_OP_PROJECT_STOP:
-      bsw_server_halt_project (BSW_SERVER, app->project);
-      bst_status_set (BST_STATUS_DONE, "Stopping Playback", NULL);
+      bst_project_ctrl_stop (BST_PROJECT_CTRL (app->pcontrols));
       break;
     case BST_OP_PROJECT_RACK_EDITOR:
       if (!app->rack_dialog)
@@ -644,70 +677,62 @@ bst_app_operate (BstApp *app,
 					    NULL);
 
 	  app->rack_editor = g_object_connect (ed, "swapped_signal::destroy", g_nullify_pointer, &app->rack_editor, NULL);
-	  bst_rack_editor_set_rack_view (ed, bsw_project_get_data_pocket (app->project, "BEAST-Rack-View"));
-	  app->rack_dialog = bst_dialog_new (&app->rack_dialog,
+	  bst_rack_editor_set_rack_view (ed, bse_project_get_data_pocket (app->project, "BEAST-Rack-View"));
+	  app->rack_dialog = gxk_dialog_new (&app->rack_dialog,
 					     GTK_OBJECT (app),
-					     0, // FIXME: undo Edit when hide && BST_DIALOG_HIDE_ON_DELETE
+					     0, // FIXME: undo Edit when hide && GXK_DIALOG_HIDE_ON_DELETE
 					     "Rack editor",
 					     app->rack_editor);
 	}
-      gtk_widget_showraise (app->rack_dialog);
+      gxk_widget_showraise (app->rack_dialog);
       break;
     case BST_OP_DIALOG_PREFERENCES:
       if (!bst_preferences)
 	{
-	  BseGConfig *gconf = bse_object_new (BST_TYPE_GCONFIG, NULL);
-	  GtkWidget *widget;
-
-	  bse_gconfig_revert (gconf);
-	  widget = bst_preferences_new (gconf);
-	  bse_object_unref (BSE_OBJECT (gconf));
-	  gtk_widget_show (widget);
-
-	  bst_preferences = bst_dialog_new (&bst_preferences,
+	  GtkWidget *widget = g_object_new (BST_TYPE_PREFERENCES,
+					    "visible", TRUE,
+					    NULL);
+	  bst_preferences = gxk_dialog_new (&bst_preferences,
 					    NULL,
-					    BST_DIALOG_HIDE_ON_DELETE,
+					    GXK_DIALOG_HIDE_ON_DELETE,
 					    "Preferences",
 					    widget);
-	  bst_preferences_create_buttons (BST_PREFERENCES (widget), BST_DIALOG (bst_preferences));
+	  bst_preferences_create_buttons (BST_PREFERENCES (widget), GXK_DIALOG (bst_preferences));
 	}
       if (!GTK_WIDGET_VISIBLE (bst_preferences))
-	bst_preferences_revert (BST_PREFERENCES (bst_dialog_get_child (BST_DIALOG (bst_preferences))));
-      gtk_widget_showraise (bst_preferences);
+	bst_preferences_revert (BST_PREFERENCES (gxk_dialog_get_child (GXK_DIALOG (bst_preferences))));
+      gxk_widget_showraise (bst_preferences);
       break;
     case BST_OP_DIALOG_PROC_BROWSER:
+#if 0 // FIXME
       if (!bst_proc_browser)
 	{
 	  GtkWidget *widget;
 
 	  widget = bst_proc_browser_new ();
 	  gtk_widget_show (widget);
-	  bst_proc_browser = bst_dialog_new (&bst_proc_browser,
+	  bst_proc_browser = gxk_dialog_new (&bst_proc_browser,
 					     NULL,
-					     BST_DIALOG_HIDE_ON_DELETE,
+					     GXK_DIALOG_HIDE_ON_DELETE,
 					     "Procedure Browser",
 					     widget);
-	  bst_proc_browser_create_buttons (BST_PROC_BROWSER (widget), BST_DIALOG (bst_proc_browser));
+	  bst_proc_browser_create_buttons (BST_PROC_BROWSER (widget), GXK_DIALOG (bst_proc_browser));
 	}
-      gtk_widget_showraise (bst_proc_browser);
+      gxk_widget_showraise (bst_proc_browser);
+#endif
       break;
     case BST_OP_DIALOG_DEVICE_MONITOR:
       any = g_object_new (BST_TYPE_SERVER_MONITOR, NULL);
       gtk_widget_show (any);
-      any = bst_dialog_new (NULL,
+      any = gxk_dialog_new (NULL,
 			    GTK_OBJECT (app),
-			    BST_DIALOG_DELETE_BUTTON, // FIXME: BST_DIALOG_HIDE_ON_DELETE && save dialog pointer
+			    GXK_DIALOG_DELETE_BUTTON, // FIXME: GXK_DIALOG_HIDE_ON_DELETE && save dialog pointer
 			    "Device Monitor",
 			    any);
       gtk_widget_show (any);
-      break;
-    case BST_OP_REFRESH:
-      gtk_container_foreach (GTK_CONTAINER (app->notebook),
-			     (GtkCallback) bst_super_shell_update,
-			     NULL);
-      gtk_widget_queue_draw (GTK_WIDGET (app->notebook));
-#if 0
-      //gsl_alloc_report ();
+      sfi_alloc_report ();
+      toyprof_report_mem ();
+#if 0 // FIXME
       {
 	GSList *slist, *olist = g_object_debug_list();
 	guint i, n_buckets = 257;
@@ -745,23 +770,31 @@ bst_app_operate (BstApp *app,
       break;
     case BST_OP_HELP_FAQ:
       help_file = g_strconcat (BST_PATH_DOCS, "/faq.markup", NULL);
-      help_title = help_file;
+      help_title = g_strdup (help_file);
       goto HELP_DIALOG;
     case BST_OP_HELP_GSL_PLAN:
       help_file = g_strconcat (BST_PATH_DOCS, "/gsl-mplan.markup", NULL);
-      help_title = help_file;
+      help_title = g_strdup (help_file);
       goto HELP_DIALOG;
     case BST_OP_HELP_QUICK_START:
       help_file = g_strconcat (BST_PATH_DOCS, "/quickstart.markup", NULL);
-      help_title = help_file;
+      help_title = g_strdup (help_file);
+      goto HELP_DIALOG;
+    case BST_OP_HELP_RELEASE_NOTES:
+      help_file = g_strconcat (BST_PATH_DOCS, "/release-notes.markup", NULL);
+      help_title = g_strdup_printf ("BEAST-%s Release Notes", BST_VERSION);
       goto HELP_DIALOG;
     HELP_DIALOG:
       if (!bst_help_dialogs[op - BST_OP_HELP_FIRST])
 	{
-	  GtkWidget *sctext = bst_scroll_text_from_file (BST_TEXT_VIEW_PARSE_TSM | BST_TEXT_VIEW_SHEET_BG, help_file);
-	  bst_help_dialogs[op - BST_OP_HELP_FIRST] = bst_dialog_new (&bst_help_dialogs[op - BST_OP_HELP_FIRST],
+	  GtkWidget *sctext = gxk_scroll_text_create (GXK_SCROLL_TEXT_NAVIGATABLE, NULL);
+	  gchar *index = g_strconcat ("file://", BST_PATH_DOCS, "/beast-index.markup", NULL);
+	  gxk_scroll_text_set_index (sctext, index);
+	  g_free (index);
+	  gxk_scroll_text_enter (sctext, help_file);
+	  bst_help_dialogs[op - BST_OP_HELP_FIRST] = gxk_dialog_new (&bst_help_dialogs[op - BST_OP_HELP_FIRST],
 								     NULL,
-								     BST_DIALOG_HIDE_ON_DELETE | BST_DIALOG_DELETE_BUTTON,
+								     GXK_DIALOG_HIDE_ON_DELETE | GXK_DIALOG_DELETE_BUTTON,
 								     help_title, sctext);
 	  g_object_set (bst_help_dialogs[op - BST_OP_HELP_FIRST],
 			"default_width", 560,
@@ -769,29 +802,9 @@ bst_app_operate (BstApp *app,
 			NULL);
 	}
       g_free (help_file);
-      gtk_widget_showraise (bst_help_dialogs[op - BST_OP_HELP_FIRST]);
-      break;
-    case BST_OP_HELP_RELEASE_NOTES:
-      if (!bst_help_dialogs[op - BST_OP_HELP_FIRST])
-	{
-	  GtkWidget *sctext;
-	  help_file = g_strconcat (BST_PATH_DOCS, "/release-notes.markup", NULL);
-	  help_title = help_file;
-	  sctext = bst_scroll_text_from_file (BST_TEXT_VIEW_PARSE_TSM | BST_TEXT_VIEW_SHEET_BG, help_file);
-	  help_file = g_strconcat (BST_PATH_DOCS, "/news.markup", NULL);
-	  bst_scroll_text_append_file_tsm (sctext, help_file);
-	  g_free (help_file);
-	  bst_help_dialogs[op - BST_OP_HELP_FIRST] = bst_dialog_new (&bst_help_dialogs[op - BST_OP_HELP_FIRST],
-								     NULL,
-								     BST_DIALOG_HIDE_ON_DELETE | BST_DIALOG_DELETE_BUTTON,
-								     help_title, sctext);
-	  g_object_set (bst_help_dialogs[op - BST_OP_HELP_FIRST],
-			"default_width", 560,
-			"default_height", 640,
-			NULL);
-	  g_free (help_title);
-	}
-      gtk_widget_showraise (bst_help_dialogs[op - BST_OP_HELP_FIRST]);
+      g_free (help_title);
+      gxk_scroll_text_rewind (gxk_dialog_get_child (GXK_DIALOG (bst_help_dialogs[op - BST_OP_HELP_FIRST])));
+      gxk_widget_showraise (bst_help_dialogs[op - BST_OP_HELP_FIRST]);
       break;
     case BST_OP_HELP_ABOUT:
       break;
@@ -801,7 +814,7 @@ bst_app_operate (BstApp *app,
       break;
     }
 
-  bst_status_window_pop ();
+  gxk_status_window_pop ();
 
   bst_update_can_operate (widget);
 
@@ -850,28 +863,22 @@ bst_app_can_operate (BstApp *app,
     {
     case BST_OP_PROJECT_NEW:
     case BST_OP_PROJECT_OPEN:
+    case BST_OP_PROJECT_MERGE:
     case BST_OP_PROJECT_SAVE_AS:
     case BST_OP_PROJECT_NEW_SONG:
     case BST_OP_PROJECT_NEW_SNET:
     case BST_OP_PROJECT_NEW_MIDI_SYNTH:
     case BST_OP_PROJECT_CLOSE:
-    case BST_OP_REFRESH:
     case BST_OP_REBUILD:
     case BST_OP_EXIT:
       return TRUE;
     case BST_OP_PROJECT_PLAY:
-      if (app->project && BSE_PROJECT (bse_object_from_id (app->project))->supers)
+      if (app->project && bse_project_can_play (app->project))
 	return TRUE;
       return FALSE;
     case BST_OP_PROJECT_STOP:
-      if (app->project)
-	{
-	  GSList *slist;
-
-	  for (slist = BSE_PROJECT (bse_object_from_id (app->project))->supers; slist; slist = slist->next)
-	    if (BSE_SOURCE_PREPARED (slist->data))
-	      return TRUE;
-	}
+      if (app->project && bse_project_is_playing (app->project))
+	return TRUE;
       return FALSE;
     case BST_OP_PROJECT_RACK_EDITOR:
     case BST_OP_DIALOG_PREFERENCES:
