@@ -18,7 +18,7 @@
 #ifndef __BST_CANVAS_SOURCE_H__
 #define __BST_CANVAS_SOURCE_H__
 
-#include	"bstdefs.h"
+#include	"bstutils.h"
 
 
 #ifdef __cplusplus
@@ -33,6 +33,7 @@ extern "C" {
 #define BST_IS_CANVAS_SOURCE(object)      (GTK_CHECK_TYPE ((object), BST_TYPE_CANVAS_SOURCE))
 #define BST_IS_CANVAS_SOURCE_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), BST_TYPE_CANVAS_SOURCE))
 #define BST_CANVAS_SOURCE_GET_CLASS(obj)  (GTK_CHECK_GET_CLASS ((obj), BST_TYPE_CANVAS_SOURCE, BstCanvasSourceClass))
+#define	BST_CANVAS_SOURCE_PIXEL_SCALE	  ((SfiReal) 100)	/* > total width/height */
 
 
 /* --- structures & typedefs --- */
@@ -42,7 +43,7 @@ struct _BstCanvasSource
 {
   GnomeCanvasGroup parent_object;
 
-  BswProxy	   source;
+  SfiProxy	   source;
 
   GtkWidget	  *source_view;
   GtkWidget	  *source_info;
@@ -68,9 +69,7 @@ struct _BstCanvasSourceClass
 /* --- prototypes --- */
 GtkType		 bst_canvas_source_get_type	(void);
 GnomeCanvasItem* bst_canvas_source_new		(GnomeCanvasGroup *group,
-						 BswProxy	   source,
-						 gdouble           world_x,
-						 gdouble           world_y);
+						 SfiProxy	   source);
 void		 bst_canvas_source_update_links	(BstCanvasSource  *csource);
 void		 bst_canvas_source_ochannel_pos	(BstCanvasSource  *csource,
 						 guint             ochannel,
