@@ -2,27 +2,22 @@
 /* Generated data (by glib-mkenums) */
 
 
-/* --- bstdefs.h --- */
-  { "BstOps", G_TYPE_ENUM, &bst__type_id__BstOps, bst_ops_values },
-
-/* --- bstdialog.h --- */
-  { "BstDialogFlags", G_TYPE_FLAGS, &bst__type_id__BstDialogFlags, bst_dialog_flags_values },
+/* --- bstmarker.h --- */
+  { "BstMarkerType", G_TYPE_ENUM, &bst__type_id__BstMarkerType, bst_marker_type_values },
 
 /* --- bstmenus.h --- */
   { "BstChoiceFlags", G_TYPE_FLAGS, &bst__type_id__BstChoiceFlags, bst_choice_flags_values },
 
-/* --- bstpianorollctrl.h --- */
-  { "BstPianoRollTool", G_TYPE_ENUM, &bst__type_id__BstPianoRollTool, bst_piano_roll_tool_values },
-
 /* --- bstqsampler.h --- */
   { "BstQSamplerType", G_TYPE_FLAGS, &bst__type_id__BstQSamplerType, bst_qsampler_type_values },
   { "BstQSamplerDrawMode", G_TYPE_ENUM, &bst__type_id__BstQSamplerDrawMode, bst_qsampler_draw_mode_values },
 
 /* --- bstradiotools.h --- */
-  { "BstRadioToolFlags", G_TYPE_FLAGS, &bst__type_id__BstRadioToolFlags, bst_radio_tool_flags_values },
+  { "BstRadioToolsFlags", G_TYPE_FLAGS, &bst__type_id__BstRadioToolsFlags, bst_radio_tools_flags_values },
 
-/* --- bsttoolbar.h --- */
-  { "BstToolbarChild", G_TYPE_ENUM, &bst__type_id__BstToolbarChild, bst_toolbar_child_values },
+/* --- bstdefs.h --- */
+  { "BstOps", G_TYPE_ENUM, &bst__type_id__BstOps, bst_ops_values },
+  { "BstQuantizationType", G_TYPE_ENUM, &bst__type_id__BstQuantizationType, bst_quantization_type_values },
 
 /* Generated data ends here */
 
