@@ -17,7 +17,6 @@
  */
 #include	"bstfiledialog.h"
 
-#include	"bststatusbar.h"
 #include	"bstmenus.h"
 #include	<unistd.h>
 #include	<errno.h>
@@ -79,30 +78,35 @@ static void
 bst_file_dialog_open (BstFileDialog *fd)
 {
   gchar *file_name;
-  BswProxy project;
+  SfiProxy project;
   BseErrorType error;
-
+  SfiProxy merge_project;
+  
   file_name = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (fd)));
+  merge_project = g_object_get_long (fd, "merger");
 
-  project = bsw_server_use_new_project (BSW_SERVER, file_name);
-  error = bsw_project_restore_from_file (project, file_name);
+  if (merge_project)
+    project = bse_item_use (merge_project);
+  else
+    project = bse_server_use_new_project (BSE_SERVER, file_name);
+  error = bse_project_restore_from_file (project, file_name);
 
   if (error)
     g_message ("failed to load project `%s': %s", /* FIXME */
 	       file_name,
 	       bse_error_blurb (error));
-  else
+  else if (!merge_project)
     {
       BstApp *app;
 
-      bsw_project_ensure_wave_repo (project);
+      bse_project_ensure_wave_repo (project);
       app = bst_app_new (project);
-      bst_status_window_push (app);
+      gxk_status_window_push (app);
       bst_status_eprintf (error,"Loading project `%s'", file_name);
-      bst_status_window_pop ();
-      gtk_idle_show_widget (GTK_WIDGET (app));
+      gxk_status_window_pop ();
+      gxk_idle_show_widget (GTK_WIDGET (app));
     }
-  bsw_item_unuse (project);
+  bse_item_unuse (project);
 
   g_free (file_name);
   gtk_widget_destroy (GTK_WIDGET (fd));
@@ -115,7 +119,7 @@ bst_file_dialog_save (BstFileDialog *fd)
   BstApp *app;
   gchar *file_name;
   GtkWidget *radio;
-  BswErrorType error;
+  BseErrorType error;
   gboolean self_contained = FALSE;
 
   file_name = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (fd)));
@@ -125,20 +129,20 @@ bst_file_dialog_save (BstFileDialog *fd)
   if (radio)
     self_contained = !GTK_TOGGLE_BUTTON (radio)->active;
 
-  bst_status_window_push (app);
+  gxk_status_window_push (app);
 
  retry_saving:
 
-  error = bsw_project_store_bse (app->project, file_name, self_contained);
+  error = bse_project_store_bse (app->project, file_name, self_contained);
 
   /* offer retry if file exists
    */
   if (error == BSE_ERROR_FILE_EXISTS)
     {
       GtkWidget *choice;
-      gchar *title = g_strdup_printf ("Saving project `%s'", bsw_item_get_name (app->project));
+      gchar *title = g_strdup_printf ("Saving project `%s'", bse_item_get_name (app->project));
       gchar *text = g_strdup_printf ("Failed to save\n`%s'\nto\n`%s':\n%s",
-				     bsw_item_get_name (app->project),
+				     bse_item_get_name (app->project),
 				     file_name,
 				     bse_error_blurb (error));
       
@@ -153,7 +157,7 @@ bst_file_dialog_save (BstFileDialog *fd)
 	{
 	  bst_choice_destroy (choice);
 	  if (unlink (file_name) < 0)
-	    bst_status_errnoprintf (errno, "Deleting `%s'", file_name);
+	    gxk_status_errnoprintf (errno, "Deleting `%s'", file_name);
 	  else
 	    goto retry_saving;
 	}
@@ -167,12 +171,13 @@ bst_file_dialog_save (BstFileDialog *fd)
 	gtk_widget_destroy (GTK_WIDGET (fd));
     }
 
-  bst_status_window_pop ();
+  gxk_status_window_pop ();
   g_free (file_name);
 }
 
 GtkWidget*
-bst_file_dialog_new_open (BstApp *app)
+bst_file_dialog_new_open (BstApp  *app,
+			  SfiProxy merge_project)
 {
   GtkWidget *dialog;
 
@@ -190,6 +195,7 @@ bst_file_dialog_new_open (BstApp *app)
   if (app)
     {
       gtk_object_set_data (GTK_OBJECT (dialog), "app", app);
+      g_object_set_long (dialog, "merger", merge_project);
       gtk_signal_connect_object_while_alive (GTK_OBJECT (app),
 					     "destroy",
 					     G_CALLBACK (gtk_widget_destroy),
@@ -211,7 +217,7 @@ bst_file_dialog_new_save (BstApp *app)
 
   g_return_val_if_fail (BST_IS_APP (app), NULL);
 
-  string = g_strconcat ("Save Project: ", bsw_item_get_name (app->project), ": BEAST", NULL);
+  string = g_strconcat ("Save Project: ", bse_item_get_name (app->project), ": BEAST", NULL);
   dialog = gtk_widget_new (BST_TYPE_FILE_DIALOG,
 			   "title", string,
 			   NULL);
