@@ -47,9 +47,9 @@
 #endif /* !G_ENABLE_DEBUG */
 
 
-/* NONE:POINTER (bstmarshal.list:25) */
+/* NONE:POINTER (./bstmarshal.list:25) */
 
-/* NONE:POINTER,POINTER (bstmarshal.list:26) */
+/* NONE:POINTER,POINTER (./bstmarshal.list:26) */
 void
 bst_marshal_VOID__POINTER_POINTER (GClosure     *closure,
                                    GValue       *return_value,
@@ -86,7 +86,7 @@ bst_marshal_VOID__POINTER_POINTER (GClosure     *closure,
             data2);
 }
 
-/* NONE:UINT,UINT (bstmarshal.list:27) */
+/* NONE:UINT,UINT (./bstmarshal.list:27) */
 void
 bst_marshal_VOID__UINT_UINT (GClosure     *closure,
                              GValue       *return_value,
@@ -123,7 +123,7 @@ bst_marshal_VOID__UINT_UINT (GClosure     *closure,
             data2);
 }
 
-/* NONE:UINT,INT (bstmarshal.list:28) */
+/* NONE:UINT,INT (./bstmarshal.list:28) */
 void
 bst_marshal_VOID__UINT_INT (GClosure     *closure,
                             GValue       *return_value,
@@ -160,7 +160,46 @@ bst_marshal_VOID__UINT_INT (GClosure     *closure,
             data2);
 }
 
-/* NONE:UINT,UINT,UINT,UINT,UINT,UINT,UINT (bstmarshal.list:29) */
+/* NONE:BOOLEAN,OBJECT (./bstmarshal.list:29) */
+void
+bst_marshal_VOID__BOOLEAN_OBJECT (GClosure     *closure,
+                                  GValue       *return_value,
+                                  guint         n_param_values,
+                                  const GValue *param_values,
+                                  gpointer      invocation_hint,
+                                  gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__BOOLEAN_OBJECT) (gpointer     data1,
+                                                     gboolean     arg_1,
+                                                     gpointer     arg_2,
+                                                     gpointer     data2);
+  register GMarshalFunc_VOID__BOOLEAN_OBJECT callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 3);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__BOOLEAN_OBJECT) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_boolean (param_values + 1),
+            g_marshal_value_peek_object (param_values + 2),
+            data2);
+}
+
+/* NONE:INT (./bstmarshal.list:30) */
+
+/* NONE:UINT,UINT,UINT,UINT,UINT,UINT,UINT (./bstmarshal.list:31) */
 void
 bst_marshal_VOID__UINT_UINT_UINT_UINT_UINT_UINT_UINT (GClosure     *closure,
                                                       GValue       *return_value,
@@ -207,9 +246,9 @@ bst_marshal_VOID__UINT_UINT_UINT_UINT_UINT_UINT_UINT (GClosure     *closure,
             data2);
 }
 
-/* NONE:UINT (bstmarshal.list:30) */
+/* NONE:UINT (./bstmarshal.list:32) */
 
-/* NONE:UINT,UINT,FLOAT (bstmarshal.list:31) */
+/* NONE:UINT,UINT,FLOAT (./bstmarshal.list:33) */
 void
 bst_marshal_VOID__UINT_UINT_FLOAT (GClosure     *closure,
                                    GValue       *return_value,
@@ -248,7 +287,7 @@ bst_marshal_VOID__UINT_UINT_FLOAT (GClosure     *closure,
             data2);
 }
 
-/* NONE:UINT,UINT,INT,BOXED (bstmarshal.list:32) */
+/* NONE:UINT,UINT,INT,BOXED (./bstmarshal.list:34) */
 void
 bst_marshal_VOID__UINT_UINT_INT_BOXED (GClosure     *closure,
                                        GValue       *return_value,
@@ -289,7 +328,7 @@ bst_marshal_VOID__UINT_UINT_INT_BOXED (GClosure     *closure,
             data2);
 }
 
-/* NONE:UINT,UINT,BOXED (bstmarshal.list:33) */
+/* NONE:UINT,UINT,BOXED (./bstmarshal.list:35) */
 void
 bst_marshal_VOID__UINT_UINT_BOXED (GClosure     *closure,
                                    GValue       *return_value,
@@ -328,7 +367,46 @@ bst_marshal_VOID__UINT_UINT_BOXED (GClosure     *closure,
             data2);
 }
 
-/* BOOL:BOOL (bstmarshal.list:34) */
+/* NONE:UINT,INT,BOXED (./bstmarshal.list:36) */
+void
+bst_marshal_VOID__UINT_INT_BOXED (GClosure     *closure,
+                                  GValue       *return_value,
+                                  guint         n_param_values,
+                                  const GValue *param_values,
+                                  gpointer      invocation_hint,
+                                  gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__UINT_INT_BOXED) (gpointer     data1,
+                                                     guint        arg_1,
+                                                     gint         arg_2,
+                                                     gpointer     arg_3,
+                                                     gpointer     data2);
+  register GMarshalFunc_VOID__UINT_INT_BOXED callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 4);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__UINT_INT_BOXED) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_uint (param_values + 1),
+            g_marshal_value_peek_int (param_values + 2),
+            g_marshal_value_peek_boxed (param_values + 3),
+            data2);
+}
+
+/* BOOL:BOOL (./bstmarshal.list:37) */
 void
 bst_marshal_BOOLEAN__BOOLEAN (GClosure     *closure,
                               GValue       *return_value,
@@ -367,9 +445,9 @@ bst_marshal_BOOLEAN__BOOLEAN (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* NONE:UINT,POINTER (bstmarshal.list:35) */
+/* NONE:UINT,POINTER (./bstmarshal.list:38) */
 
-/* NONE:OBJECT,OBJECT (bstmarshal.list:36) */
+/* NONE:OBJECT,OBJECT (./bstmarshal.list:39) */
 void
 bst_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
                                  GValue       *return_value,
@@ -406,11 +484,11 @@ bst_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
             data2);
 }
 
-/* NONE:STRING (bstmarshal.list:37) */
+/* NONE:STRING (./bstmarshal.list:40) */
 
-/* NONE:BOOLEAN (bstmarshal.list:38) */
+/* NONE:BOOLEAN (./bstmarshal.list:41) */
 
-/* BOOLEAN:UINT (bstmarshal.list:39) */
+/* BOOLEAN:UINT (./bstmarshal.list:42) */
 void
 bst_marshal_BOOLEAN__UINT (GClosure     *closure,
                            GValue       *return_value,
@@ -449,5 +527,5 @@ bst_marshal_BOOLEAN__UINT (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* NONE:BOXED (bstmarshal.list:40) */
+/* NONE:BOXED (./bstmarshal.list:43) */
 
