@@ -18,15 +18,14 @@
  */
 #include "bstmenus.h"
 
-#include "bstdialog.h"
-
+#include <string.h>
 
 /* --- structures --- */
 struct _BstChoice
 {
   BstChoiceFlags type_and_flags;
   const gchar   *icon_stock_id;
-  BswIcon       *bsw_icon;
+  BseIcon       *bse_icon;
   const gchar   *name;
   gpointer       p_id;
 };
@@ -36,9 +35,10 @@ struct _BstChoice
 
 
 /* --- variables --- */
-static gboolean   main_quit_on_menu_item_activate = FALSE;
+static gboolean   modal_loop_running = FALSE;
+static gboolean   modal_loop_quit_on_menu_item_activate = FALSE;
 static GtkWidget *current_popup_menu = NULL;
-static gchar     *bstmenu_icon_item = "<BstMenuIconItem>";
+static gchar     *bstmenu_category_item = "<BstMenuCategoryItem>";
 
 
 /* --- functions --- */
@@ -46,77 +46,123 @@ static gint
 menu_entries_compare (gconstpointer a,
 		      gconstpointer b)
 {
-  const GtkItemFactoryEntry *entry_a = a;
-  const GtkItemFactoryEntry *entry_b = b;
+  const BstMenuConfigEntry *entry_a = a;
+  const BstMenuConfigEntry *entry_b = b;
 
   return strcmp (entry_a->path, entry_b->path);
 }
 
-GSList*
-bst_menu_entries_sort (GSList *entry_slist)
+void
+bst_menu_config_sort (BstMenuConfig *config)
 {
-  GSList *slist, *branch_slist = NULL, *item_slist = NULL;
+  SfiRing *entry, *branches = NULL, *items = NULL;
 
-  for (slist = entry_slist; slist; slist = slist->next)
+  for (entry = config->entries; entry; entry = sfi_ring_walk (entry, config->entries))
     {
-      GtkItemFactoryEntry *entry = slist->data;
+      BstMenuConfigEntry *e = entry->data;
 
-      if (!entry->item_type || !entry->item_type[0] ||
-	  g_pattern_match_simple ("<*Item>", entry->item_type))
-	item_slist = g_slist_prepend (item_slist, entry);
+      if (!e->item_type || !e->item_type[0] || g_pattern_match_simple ("<*Item>", e->item_type))
+	items = sfi_ring_append (items, e);
       else
-	branch_slist = g_slist_prepend (branch_slist, entry);
+	branches = sfi_ring_append (branches, e);
     }
-  g_slist_free (entry_slist);
-
-  return g_slist_concat (g_slist_reverse (branch_slist),
-			 g_slist_sort (item_slist, menu_entries_compare));
+  config->entries = sfi_ring_concat (branches, sfi_ring_sort (items, menu_entries_compare));
 }
 
-GtkItemFactoryEntry*
-bst_menu_entries_from_cats (guint              n_cats,
-			    const BseCategory *cats,
-			    BstMenuCallback    callback,
-			    gboolean	       remove_toplevel)
+void
+bst_menu_config_reverse (BstMenuConfig *config)
 {
-  GtkItemFactoryEntry *entries, *entry;
+  config->entries = sfi_ring_reverse (config->entries);
+}
 
-  if (!n_cats)
-    return NULL;
-  g_return_val_if_fail (cats != NULL, NULL);
-  g_return_val_if_fail (callback != NULL, NULL);
-  
-  entries = g_new0 (GtkItemFactoryEntry, n_cats);
-  for (entry = entries; entry < entries + n_cats; entry++)
+static void
+menu_config_append (BstMenuConfig      *config,
+		    BstMenuConfigEntry *entry)
+{
+  BstMenuConfigEntry *e = g_new0 (BstMenuConfigEntry, 1);
+  config->gcentries = g_slist_prepend (config->gcentries, e);
+  e->path = g_strdup (entry->path);
+  e->accelerator = g_strdup (entry->accelerator);
+  e->callback = entry->callback;
+  e->callback_action = entry->callback_action;
+  e->item_type = g_strdup (entry->item_type);
+  e->extra_data = entry->extra_data;
+  config->entries = sfi_ring_append (config->entries, e);
+}
+
+static BseIcon*
+menu_config_copy_icon (BstMenuConfig *config,
+		       BseIcon       *icon)
+{
+  BseIcon *ic = bse_icon_copy_shallow (icon);
+  config->gcicons = g_slist_prepend (config->gcicons, ic);
+  return ic;
+}
+
+BstMenuConfig*
+bst_menu_config_from_cats (BseCategorySeq *cseq,
+			   BstMenuCatFunc  callback,
+			   guint           skip_levels)
+{
+  BstMenuConfig *config = g_new0 (BstMenuConfig, 1);
+  guint i;
+
+  for (i = 0; i < cseq->n_cats; i++)
     {
-      entry->path = cats->category + (remove_toplevel ? cats->mindex : 0);
-      entry->accelerator = NULL;
-      entry->callback = callback;
-      entry->callback_action = cats->type;
-      entry->item_type = bstmenu_icon_item;
-      entry->extra_data = cats->icon;
-      cats++;
+      BstMenuConfigEntry e = { 0, };
+      e.path = cseq->cats[i]->category + (skip_levels ? cseq->cats[i]->mindex : 0);
+      e.accelerator = NULL;
+      e.callback = (BstMenuUserFunc) callback;
+      e.callback_action = cseq->cats[i]->category_id;
+      e.item_type = bstmenu_category_item;
+      e.extra_data = menu_config_copy_icon (config, cseq->cats[i]->icon);
+      menu_config_append (config, &e);
     }
-  
-  return entries;
+
+  return config;
 }
 
-GSList*
-bst_menu_entries_slist (guint                      n_ientries,
-			const GtkItemFactoryEntry *ientries)
+BstMenuConfig*
+bst_menu_config_from_entries (guint               n_entries,
+			      BstMenuConfigEntry *entries)
 {
-  GtkItemFactoryEntry *entry = (GtkItemFactoryEntry*) ientries;
-  GSList *entry_slist = NULL;
+  BstMenuConfig *config = g_new0 (BstMenuConfig, 1);
+  guint i;
 
-  if (!n_ientries)
-    return NULL;
-  g_return_val_if_fail (ientries != NULL, NULL);
+  for (i = 0; i < n_entries; i++)
+    menu_config_append (config, entries + i);
+  return config;
+}
 
-  entry += n_ientries - 1;
-  while (entry >= ientries)
-    entry_slist = g_slist_prepend (entry_slist, entry--);
+BstMenuConfig*
+bst_menu_config_merge (BstMenuConfig *config,
+		       BstMenuConfig *merge_config)
+{
+  config->entries = sfi_ring_concat (config->entries, merge_config->entries);
+  config->gcentries = g_slist_concat (config->gcentries, merge_config->gcentries);
+  config->gcicons = g_slist_concat (config->gcicons, merge_config->gcicons);
+  g_free (merge_config);
+  return config;
+}
 
-  return entry_slist;
+void
+bst_menu_config_free (BstMenuConfig *config)
+{
+  GSList *node;
+  for (node = config->gcentries; node; node = node->next)
+    {
+      BstMenuConfigEntry *e = node->data;
+      g_free (e->path);
+      g_free (e->accelerator);
+      g_free (e->item_type);
+      g_free (e);
+    }
+  for (node = config->gcicons; node; node = node->next)
+    bse_icon_free (node->data);
+  sfi_ring_free (config->entries);
+  g_slist_free (config->gcentries);
+  g_slist_free (config->gcicons);
+  g_free (config);
 }
 
 typedef struct {
@@ -131,9 +177,9 @@ bst_menu_item_wrapper (GtkWidget *menu_item,
 {
   GtkItemFactory *ifactory = gtk_item_factory_from_widget (menu_item);
   GtkWidget *owner = g_object_get_data (G_OBJECT (menu_item), "bst-menu-owner");
-  gpointer callback_action = g_object_get_data (G_OBJECT (menu_item), "bst-menu-callback-action");
+  gulong callback_action = (gulong) g_object_get_data (G_OBJECT (menu_item), "bst-menu-callback-action");
   PopupData *pdata = gtk_item_factory_popup_data (ifactory);
-  BstMenuCallback callback = data;
+  BstMenuCatFunc callback = data;
 
   if (!owner)
     owner = pdata ? pdata->owner : NULL;
@@ -166,60 +212,72 @@ bst_menu_item_wrapper (GtkWidget *menu_item,
     }
 
   if (owner)
-    callback (owner, (gulong) callback_action, pdata ? pdata->popup_data : NULL);
+    callback (owner, callback_action, pdata ? pdata->popup_data : NULL);
   else
     gdk_beep ();
 }
 
 void
-bst_menu_entries_create (GtkItemFactory *ifactory,
-			 GSList         *bst_menu_entries,
-			 GtkWidget	*owner)
+bst_menu_config_create_items (BstMenuConfig  *config,
+			      GtkItemFactory *ifactory,
+			      GtkWidget      *owner)
 {
-  GSList *slist;
-  
+  SfiRing *ring;
+
   g_return_if_fail (GTK_IS_ITEM_FACTORY (ifactory));
   if (!GTK_IS_MENU (ifactory->widget))
     g_return_if_fail (GTK_IS_WIDGET (owner));
 
-  for (slist = bst_menu_entries; slist; slist = slist->next)
+  for (ring = config->entries; ring; ring = sfi_ring_walk (ring, config->entries))
     {
-      GtkItemFactoryEntry entry = { 0, }, *ientry = slist->data;
+      GtkItemFactoryEntry ife = { 0, };
+      BstMenuConfigEntry *e = ring->data;
       GtkWidget *item;
 
       /* create menu item */
-      entry.path = ientry->path;
-      entry.accelerator = ientry->accelerator;
-      entry.callback = NULL;
-      entry.callback_action = 0;
-      entry.item_type = ientry->item_type == bstmenu_icon_item ? "<ImageItem>" : ientry->item_type;
-      entry.extra_data = ientry->item_type == bstmenu_icon_item ? "!Pix" : ientry->extra_data;
-      gtk_item_factory_create_items (ifactory, 1, &entry, NULL);
-      item = gtk_item_factory_get_item (ifactory, entry.path);
+      ife.path = e->path;
+      ife.accelerator = e->accelerator;
+      ife.callback = NULL;
+      ife.callback_action = 0;
+      if (e->item_type && strcmp (e->item_type, bstmenu_category_item) == 0)
+	{
+	  ife.item_type = "<ImageItem>";
+	  /* we want an image menu item without an image, which gtk
+	   * doesn't properly support, so we just provide an invalid
+	   * serialized pixbuf (via magic mismatch)
+	   */
+	  ife.extra_data = "!Pix";
+	}
+      else
+	{
+	  ife.item_type = e->item_type;
+	  ife.extra_data = e->extra_data;
+	}
+      gtk_item_factory_create_items (ifactory, 1, &ife, NULL);
+      item = gtk_item_factory_get_item (ifactory, ife.path);
 
       /* connect callback */
-      if (ientry->callback)
+      if (e->callback)
 	{
 	  g_object_set_data (G_OBJECT (item), "bst-menu-owner", owner);
-	  g_object_set_data (G_OBJECT (item), "bst-menu-callback", ientry->callback);
-	  g_object_set_data (G_OBJECT (item), "bst-menu-callback-action", (void*) ientry->callback_action);
+	  // g_object_set_data (G_OBJECT (item), "bst-menu-callback", e->callback);
+	  g_object_set_data (G_OBJECT (item), "bst-menu-callback-action", (void*) e->callback_action);
 	  gtk_signal_connect (GTK_OBJECT (item),
 			      "activate",
 			      GTK_SIGNAL_FUNC (bst_menu_item_wrapper),
-			      ientry->callback);
+			      e->callback);
 	}
 
-      /* create image from BswIcon */
-      if (ientry->item_type == bstmenu_icon_item)
+      /* create image from BseIcon */
+      if (e->item_type == bstmenu_category_item)
 	{
-	  GtkWidget *image = bst_image_from_icon ((BswIcon*) ientry->extra_data, BST_SIZE_MENU);
-
+	  GtkWidget *image = bst_image_from_icon ((BseIcon*) e->extra_data, BST_SIZE_MENU);
 	  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
 	}
 
       /* fixup Title items */
-      if (entry.item_type && strcmp (entry.item_type, "<Title>") == 0)
-	bst_widget_modify_as_title (GTK_BIN (item)->child);
+      if (ife.item_type && strcmp (ife.item_type, "<Title>") == 0)
+	gxk_widget_modify_as_title (GTK_BIN (item)->child);
     }
 }
 
@@ -267,9 +325,9 @@ bst_menu_add_accel_owner (GtkItemFactory  *ifactory,
   g_return_if_fail (GTK_IS_ITEM_FACTORY (ifactory));
   g_return_if_fail (GTK_IS_WIDGET (owner));
 
-  slist = g_object_get_data (G_OBJECT (ifactory), "bst-menu-accel-owners");
+  slist = g_object_steal_data (G_OBJECT (ifactory), "bst-menu-accel-owners");
   slist = g_slist_prepend (slist, owner);
-  g_object_set_data (G_OBJECT (ifactory), "bst-menu-accel-owners", slist);
+  g_object_set_data_full (G_OBJECT (ifactory), "bst-menu-accel-owners", slist, (GDestroyNotify) g_slist_free);
 }
 
 BstChoice*
@@ -277,13 +335,13 @@ bst_choice_alloc (BstChoiceFlags type,
 		  const gchar   *choice_name,
 		  gpointer       choice_id,
 		  const gchar   *icon_stock_id,
-		  BswIcon       *icon)
+		  BseIcon       *icon)
 {
   BstChoice *choice = g_new (BstChoice, 1);
 
   choice->type_and_flags = type;
   choice->icon_stock_id = icon_stock_id;
-  choice->bsw_icon = icon ? bsw_icon_ref (icon) : NULL;
+  choice->bse_icon = icon ? bse_icon_copy_shallow (icon) : NULL;
   choice->name = choice_name;
   choice->p_id = choice_id;
 
@@ -300,8 +358,8 @@ menu_choice_activate (GtkWidget *item,
     {
       gtk_object_set_data (GTK_OBJECT (current_popup_menu), "BstChoice", udata);
       
-      if (main_quit_on_menu_item_activate)
-	gtk_main_quit ();
+      if (modal_loop_quit_on_menu_item_activate)
+	modal_loop_running = FALSE;
     }
   else	/* current_popup_menu is not set e.g. for option menus */
     {
@@ -332,10 +390,10 @@ button_choice_activate (GtkWidget *item,
 }
 
 static void
-check_main_quit (GtkWidget *item)
+check_modal_quit (GtkWidget *item)
 {
-  if (main_quit_on_menu_item_activate)
-    gtk_main_quit ();
+  if (modal_loop_quit_on_menu_item_activate)
+    modal_loop_running = FALSE;
 }
 
 static void
@@ -355,8 +413,8 @@ menu_item_add_activator (GtkWidget *widget,
 static void
 free_choice (BstChoice *choice)
 {
-  if (choice->bsw_icon)
-    bsw_icon_unref (choice->bsw_icon);
+  if (choice->bse_icon)
+    bse_icon_free (choice->bse_icon);
   g_free (choice);
 }
 
@@ -391,7 +449,7 @@ bst_choice_menu_add_choice_and_free (GtkWidget *menu,
 
       if (choice->icon_stock_id)
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
-				       bst_image_from_stock (choice->icon_stock_id, BST_SIZE_MENU));
+				       gxk_stock_image (choice->icon_stock_id, BST_SIZE_MENU));
       any = gtk_widget_new (GTK_TYPE_ACCEL_LABEL,
 			    "visible", TRUE,
 			    "label", choice->name,
@@ -400,7 +458,7 @@ bst_choice_menu_add_choice_and_free (GtkWidget *menu,
 			    "xalign", 0.0,
 			    NULL);
       if (choice_type == BST_CHOICE_TYPE_TITLE)
-	bst_widget_modify_as_title (any);
+	gxk_widget_modify_as_title (any);
     }
   free_choice (choice);
 }
@@ -442,7 +500,7 @@ bst_choice_menu_createv (const gchar *menu_path,
 
   menu = g_object_connect (gtk_widget_new (GTK_TYPE_MENU,
 					   NULL),
-			   "signal::selection-done", check_main_quit, NULL,
+			   "signal::selection-done", check_modal_quit, NULL,
 			   NULL);
   gtk_menu_set_accel_path (GTK_MENU (menu), menu_path);
   gtk_widget_ref (menu);
@@ -514,7 +572,7 @@ bst_choice_dialog_createv (BstChoice *first_choice,
   
   /* create dialog
    */
-  dialog = bst_dialog_new (NULL, NULL, BST_DIALOG_POPUP_POS | BST_DIALOG_MODAL,
+  dialog = gxk_dialog_new (NULL, NULL, GXK_DIALOG_POPUP_POS | GXK_DIALOG_MODAL,
 			   NULL, vbox);
   gtk_widget_ref (dialog);
   gtk_object_sink (GTK_OBJECT (dialog));
@@ -535,10 +593,10 @@ bst_choice_dialog_createv (BstChoice *first_choice,
 	  gtk_widget_set (dialog, "title", choice->name, NULL);
 	  break;
 	case BST_CHOICE_TYPE_ITEM:
-	  any = bst_dialog_action_multi (BST_DIALOG (dialog), choice->name,
+	  any = gxk_dialog_action_multi (GXK_DIALOG (dialog), choice->name,
 					 button_choice_activate, choice->p_id,
 					 choice->icon_stock_id,
-					 (choice_flags & BST_CHOICE_FLAG_DEFAULT) ? BST_DIALOG_MULTI_DEFAULT : 0);
+					 (choice_flags & BST_CHOICE_FLAG_DEFAULT) ? GXK_DIALOG_MULTI_DEFAULT : 0);
 	  if (choice_flags & BST_CHOICE_FLAG_INSENSITIVE)
 	    gtk_widget_set_sensitive (any, FALSE);
 	  break;
@@ -586,9 +644,9 @@ bst_choice_selectable (GtkWidget *widget)
 	}
       g_list_free (children);
     }
-  else if (BST_IS_DIALOG (widget))
+  else if (GXK_IS_DIALOG (widget))
     {
-      GList *list, *children = gtk_container_children (GTK_CONTAINER (BST_DIALOG (widget)->hbox));
+      GList *list, *children = gtk_container_children (GTK_CONTAINER (GXK_DIALOG (widget)->hbox));
 
       for (list = children; list; list = list->next)
 	{
@@ -621,17 +679,23 @@ bst_choice_modal (GtkWidget *choice,
 
       if (bst_choice_selectable (choice))
 	{
-	  main_quit_on_menu_item_activate = TRUE;
+	  modal_loop_quit_on_menu_item_activate = TRUE;
+	  modal_loop_running = TRUE;
 	  current_popup_menu = GTK_WIDGET (menu);
 	  gtk_menu_popup (menu, NULL, NULL, NULL, NULL, mouse_button, time);
-	  gtk_main ();
+	  while (modal_loop_running)
+	    {
+	      GDK_THREADS_LEAVE ();
+	      g_main_iteration (TRUE);
+	      GDK_THREADS_ENTER ();
+	    }
 	  current_popup_menu = NULL;
-	  main_quit_on_menu_item_activate = FALSE;
+	  modal_loop_quit_on_menu_item_activate = FALSE;
 	}
       
       data = gtk_object_get_data (GTK_OBJECT (menu), "BstChoice");
     }
-  else if (BST_IS_DIALOG (choice))
+  else if (GXK_IS_DIALOG (choice))
     {
       gtk_object_set_data (GTK_OBJECT (choice), "BstChoice", data);
 
@@ -639,13 +703,12 @@ bst_choice_modal (GtkWidget *choice,
 	{
 	  gtk_widget_show (choice);
 
-	  do
+          while (GTK_WIDGET_VISIBLE (choice))
 	    {
 	      GDK_THREADS_LEAVE ();
 	      g_main_iteration (TRUE);
 	      GDK_THREADS_ENTER ();
 	    }
-	  while (GTK_WIDGET_VISIBLE (choice));
 	}
 
       data = gtk_object_get_data (GTK_OBJECT (choice), "BstChoice");
