@@ -17,7 +17,6 @@
  */
 #include	"bstpartdialog.h"
 
-#include	"bststatusbar.h"
 #include	"bstprocedure.h"
 #include	"bstmenus.h"
 
@@ -27,23 +26,25 @@ static void	bst_part_dialog_class_init	(BstPartDialogClass	*klass);
 static void	bst_part_dialog_init		(BstPartDialog		*part_dialog);
 static void	bst_part_dialog_finalize	(GObject		*object);
 static void	part_dialog_update_tool		(BstPartDialog		*part_dialog);
-static void	piano_canvas_press		(BstPartDialog		*part_dialog,
+static void	piano_canvas_clicked		(BstPartDialog		*part_dialog,
 						 guint			 button,
 						 guint			 tick_position,
 						 gint			 note,
 						 GdkEvent		*event,
 						 BstPianoRoll		*proll);
 static void	part_dialog_run_proc		(GtkWidget		*widget,
-						 gulong			 callback_action,
+						 gulong                  category_id,
 						 gpointer		 popup_data);
 static void	part_dialog_note_choice		(BstPartDialog		*self,
 						 guint			 choice);
 static void	part_dialog_qnote_choice	(BstPartDialog		*self,
 						 guint			 choice);
-static void	menu_select_tool		(BstPartDialog		*self,
-						 guint			 tool);
-static void	menu_activate_tool		(BstPartDialog		*self,
-						 guint			 tool);
+static void	menu_select_tool		(GtkWidget              *owner,
+						 gulong                  callback_action,
+						 gpointer                popup_data);
+static void	menu_activate_tool		(GtkWidget              *owner,
+						 gulong                  callback_action,
+						 gpointer                popup_data);
 
 
 /* --- variables --- */
@@ -53,7 +54,7 @@ enum {
   ACTION_COPY,
   ACTION_PASTE
 };
-static GtkItemFactoryEntry popup_entries[] =
+static BstMenuConfigEntry popup_entries[] =
 {
 #define MENU_CB(xxx)	menu_select_tool, BST_PIANO_ROLL_TOOL_ ## xxx
 #define ACTION_CB(xxx)	menu_activate_tool, ACTION_ ## xxx
@@ -94,7 +95,7 @@ bst_part_dialog_get_type (void)
 	(GInstanceInitFunc) bst_part_dialog_init,
       };
       
-      type = g_type_register_static (BST_TYPE_DIALOG,
+      type = g_type_register_static (GXK_TYPE_DIALOG,
 				     "BstPartDialog",
 				     &type_info, 0);
     }
@@ -106,11 +107,9 @@ static void
 bst_part_dialog_class_init (BstPartDialogClass *class)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (class);
-  GtkItemFactoryEntry *centries;
-  BseCategory *cats;
-  GSList *slist;
-  guint n_cats;
-  
+  BseCategorySeq *cseq;
+  BstMenuConfig *m1, *m2;
+
   parent_class = g_type_class_peek_parent (class);
   
   gobject_class->finalize = bst_part_dialog_finalize;
@@ -118,18 +117,17 @@ bst_part_dialog_class_init (BstPartDialogClass *class)
   /* create item factory for menu entries and categories */
   class->popup_factory = gtk_item_factory_new (GTK_TYPE_MENU, "<BstPartDialog>", NULL);
 
-  /* construct menu entry list */
-  cats = bse_categories_match_typed ("/Scripts/*", BSE_TYPE_PROCEDURE, &n_cats);
-  centries = bst_menu_entries_from_cats (n_cats, cats, part_dialog_run_proc, TRUE);
-  slist = bst_menu_entries_slist (n_cats, centries);
-  slist = bst_menu_entries_sort (slist);
-  slist = g_slist_concat (bst_menu_entries_slist (G_N_ELEMENTS (popup_entries), popup_entries), slist);
-
-  /* create entries and release allocations */
-  bst_menu_entries_create (class->popup_factory, slist, NULL);
-  g_slist_free (slist);
-  g_free (centries);
-  g_free (cats);
+  /* standard entries */
+  m1 = bst_menu_config_from_entries (G_N_ELEMENTS (popup_entries), popup_entries);
+  /* procedures */
+  cseq = bse_categories_match_typed ("/Scripts/*", "BseProcedure");
+  m2 = bst_menu_config_from_cats (cseq, part_dialog_run_proc, 1);
+  bst_menu_config_sort (m2);
+  /* merge */
+  m1 = bst_menu_config_merge (m1, m2);
+  /* and create menu items */
+  bst_menu_config_create_items (m1, class->popup_factory, NULL);
+  bst_menu_config_free (m1);
 }
 
 static void
@@ -159,12 +157,12 @@ bst_part_dialog_init (BstPartDialog *self)
   g_object_set (self,
 		"default_width", 600,
 		"default_height", 450,
-		"flags", BST_DIALOG_STATUS_SHELL,
+		"flags", GXK_DIALOG_STATUS_SHELL,
 		NULL);
-  main_vbox = BST_DIALOG (self)->vbox;
+  main_vbox = GXK_DIALOG (self)->vbox;
 
   /* create toolbar */
-  self->toolbar = bst_toolbar_new (&self->toolbar);
+  self->toolbar = gxk_toolbar_new (&self->toolbar);
   gtk_box_pack_start (GTK_BOX (main_vbox), GTK_WIDGET (self->toolbar), FALSE, TRUE, 0);
 
   /* create scrolled window */
@@ -187,13 +185,11 @@ bst_part_dialog_init (BstPartDialog *self)
   self->proll_ctrl = bst_piano_roll_controller_new (BST_PIANO_ROLL (self->proll));
   g_object_connect (self->proll,
 		    "swapped_signal::destroy", g_nullify_pointer, &self->proll,
-		    "swapped_signal::canvas_press", piano_canvas_press, self,
+		    "swapped_signal::canvas-clicked", piano_canvas_clicked, self,
 		    NULL);
 
   /* radio tools */
   self->rtools = bst_radio_tools_new ();
-  g_object_ref (self->rtools);
-  gtk_object_sink (GTK_OBJECT (self->rtools));
   g_object_connect (self->rtools,
 		    "swapped_signal::set_tool", part_dialog_update_tool, self,
 		    NULL);
@@ -215,49 +211,49 @@ bst_part_dialog_init (BstPartDialog *self)
   bst_radio_tools_build_toolbar (self->rtools, self->toolbar);
 
   /* selection ops (copy/cut/...) */
-  bst_toolbar_append_separator (self->toolbar);
-  button = bst_toolbar_append_stock (self->toolbar, BST_TOOLBAR_BUTTON, "Clear", "Clear the current selection", BST_STOCK_TRASH_SCISSORS);
+  gxk_toolbar_append_separator (self->toolbar);
+  button = gxk_toolbar_append_stock (self->toolbar, GXK_TOOLBAR_BUTTON, "Clear", "Clear the current selection", BST_STOCK_TRASH_SCISSORS);
   g_object_connect (button, "swapped_signal::clicked", bst_piano_roll_controller_clear, self->proll_ctrl, NULL);
-  button = bst_toolbar_append_stock (self->toolbar, BST_TOOLBAR_BUTTON, "Cut", "Move the current selection into clipboard", BST_STOCK_MUSIC_CUT);
+  button = gxk_toolbar_append_stock (self->toolbar, GXK_TOOLBAR_BUTTON, "Cut", "Move the current selection into clipboard", BST_STOCK_MUSIC_CUT);
   g_object_connect (button, "swapped_signal::clicked", bst_piano_roll_controller_cut, self->proll_ctrl, NULL);
-  button = bst_toolbar_append_stock (self->toolbar, BST_TOOLBAR_BUTTON, "Copy", "Copy the current selection into clipboard", BST_STOCK_MUSIC_COPY);
+  button = gxk_toolbar_append_stock (self->toolbar, GXK_TOOLBAR_BUTTON, "Copy", "Copy the current selection into clipboard", BST_STOCK_MUSIC_COPY);
   g_object_connect (button, "swapped_signal::clicked", bst_piano_roll_controller_copy, self->proll_ctrl, NULL);
-  button = bst_toolbar_append_stock (self->toolbar, BST_TOOLBAR_BUTTON, "Paste", "Insert clipboard contents as current selection", BST_STOCK_MUSIC_PASTE);
+  button = gxk_toolbar_append_stock (self->toolbar, GXK_TOOLBAR_BUTTON, "Paste", "Insert clipboard contents as current selection", BST_STOCK_MUSIC_PASTE);
   g_object_connect (button, "swapped_signal::clicked", bst_piano_roll_controller_paste, self->proll_ctrl, NULL);
 
   /* note selection */
-  bst_toolbar_append_separator (self->toolbar);
-  choice = bst_toolbar_append_choice (self->toolbar, BST_TOOLBAR_TRUNC_BUTTON,
-				      (BstToolbarChoiceFunc) part_dialog_note_choice, self, NULL);
-  bst_toolbar_choice_add (choice, "1/1", "Insert full notes",
-			  bst_image_from_stock (BST_STOCK_NOTE_1, BST_SIZE_TOOLBAR), 1);
-  bst_toolbar_choice_add (choice, "1/2", "Insert half notes",
-			  bst_image_from_stock (BST_STOCK_NOTE_2, BST_SIZE_TOOLBAR), 2);
-  bst_toolbar_choice_set (choice, "1/4", "Insert quarter notes",
-			  bst_image_from_stock (BST_STOCK_NOTE_4, BST_SIZE_TOOLBAR), 4);
-  bst_toolbar_choice_add (choice, "1/8", "Insert eighths note",
-			  bst_image_from_stock (BST_STOCK_NOTE_8, BST_SIZE_TOOLBAR), 8);
-  bst_toolbar_choice_add (choice, "1/16", "Insert sixteenth note",
-			  bst_image_from_stock (BST_STOCK_NOTE_16, BST_SIZE_TOOLBAR), 16);
+  gxk_toolbar_append_separator (self->toolbar);
+  choice = gxk_toolbar_append_choice (self->toolbar, GXK_TOOLBAR_TRUNC_BUTTON,
+				      (GxkToolbarChoiceFunc) part_dialog_note_choice, self, NULL);
+  gxk_toolbar_choice_add (choice, "1/1", "Insert full notes",
+			  gxk_stock_image (BST_STOCK_NOTE_1, BST_SIZE_TOOLBAR), 1);
+  gxk_toolbar_choice_add (choice, "1/2", "Insert half notes",
+			  gxk_stock_image (BST_STOCK_NOTE_2, BST_SIZE_TOOLBAR), 2);
+  gxk_toolbar_choice_set (choice, "1/4", "Insert quarter notes",
+			  gxk_stock_image (BST_STOCK_NOTE_4, BST_SIZE_TOOLBAR), 4);
+  gxk_toolbar_choice_add (choice, "1/8", "Insert eighths note",
+			  gxk_stock_image (BST_STOCK_NOTE_8, BST_SIZE_TOOLBAR), 8);
+  gxk_toolbar_choice_add (choice, "1/16", "Insert sixteenth note",
+			  gxk_stock_image (BST_STOCK_NOTE_16, BST_SIZE_TOOLBAR), 16);
   
   /* quantization selection */
-  choice = bst_toolbar_append_choice (self->toolbar, BST_TOOLBAR_TRUNC_BUTTON,
-				      (BstToolbarChoiceFunc) part_dialog_qnote_choice, self, NULL);
-  bst_toolbar_choice_add (choice, "None", "No quantization selected",
-			  bst_image_from_stock (BST_STOCK_QNOTE_NONE, BST_SIZE_TOOLBAR), 0);
-  bst_toolbar_choice_add (choice, "Q: 1/1", "Quantize to full note boundaries",
-			  bst_image_from_stock (BST_STOCK_QNOTE_1, BST_SIZE_TOOLBAR), 1);
-  bst_toolbar_choice_add (choice, "Q: 1/2", "Quantize to half note boundaries",
-			  bst_image_from_stock (BST_STOCK_QNOTE_2, BST_SIZE_TOOLBAR), 2);
-  bst_toolbar_choice_add (choice, "Q: 1/4", "Quantize to quarter note boundaries",
-			  bst_image_from_stock (BST_STOCK_QNOTE_4, BST_SIZE_TOOLBAR), 4);
-  bst_toolbar_choice_set (choice, "Q: 1/8", "Quantize to eighths note boundaries",
-			  bst_image_from_stock (BST_STOCK_QNOTE_8, BST_SIZE_TOOLBAR), 8);
-  bst_toolbar_choice_add (choice, "Q: 1/16", "Quantize to sixteenth note boundaries",
-			  bst_image_from_stock (BST_STOCK_QNOTE_16, BST_SIZE_TOOLBAR), 16);
+  choice = gxk_toolbar_append_choice (self->toolbar, GXK_TOOLBAR_TRUNC_BUTTON,
+				      (GxkToolbarChoiceFunc) part_dialog_qnote_choice, self, NULL);
+  gxk_toolbar_choice_add (choice, "None", "No quantization selected",
+			  gxk_stock_image (BST_STOCK_QNOTE_NONE, BST_SIZE_TOOLBAR), 0);
+  gxk_toolbar_choice_add (choice, "Q: 1/1", "Quantize to full note boundaries",
+			  gxk_stock_image (BST_STOCK_QNOTE_1, BST_SIZE_TOOLBAR), 1);
+  gxk_toolbar_choice_add (choice, "Q: 1/2", "Quantize to half note boundaries",
+			  gxk_stock_image (BST_STOCK_QNOTE_2, BST_SIZE_TOOLBAR), 2);
+  gxk_toolbar_choice_add (choice, "Q: 1/4", "Quantize to quarter note boundaries",
+			  gxk_stock_image (BST_STOCK_QNOTE_4, BST_SIZE_TOOLBAR), 4);
+  gxk_toolbar_choice_set (choice, "Q: 1/8", "Quantize to eighths note boundaries",
+			  gxk_stock_image (BST_STOCK_QNOTE_8, BST_SIZE_TOOLBAR), 8);
+  gxk_toolbar_choice_add (choice, "Q: 1/16", "Quantize to sixteenth note boundaries",
+			  gxk_stock_image (BST_STOCK_QNOTE_16, BST_SIZE_TOOLBAR), 16);
 
   /* vzoom */
-  bst_toolbar_append_separator (self->toolbar);
+  gxk_toolbar_append_separator (self->toolbar);
   adjustment = gtk_adjustment_new (4, 1, 16, 1, 4, 0);
   g_object_connect (adjustment,
 		    "swapped_signal_after::value_changed", vzoom_changed, self,
@@ -266,13 +262,13 @@ bst_part_dialog_init (BstPartDialog *self)
 			"visible", TRUE,
 			"adjustment", adjustment,
 			"digits", 0,
-			"width_request", 2 * bst_size_width (BST_SIZE_TOOLBAR),
+			"width_request", 2 * gxk_size_width (BST_SIZE_TOOLBAR),
 			NULL);
-  bst_toolbar_append (self->toolbar, BST_TOOLBAR_EXTRA_WIDGET,
+  gxk_toolbar_append (self->toolbar, GXK_TOOLBAR_EXTRA_WIDGET,
 		      "VZoom", "Vertical Zoom", entry);
 
   /* hzoom */
-  // bst_toolbar_append_space (self->toolbar);
+  // gxk_toolbar_append_space (self->toolbar);
   adjustment = gtk_adjustment_new (13, 0, 100, 1, 5, 0);
   g_object_connect (adjustment,
 		    "swapped_signal_after::value_changed", hzoom_changed, self,
@@ -281,9 +277,9 @@ bst_part_dialog_init (BstPartDialog *self)
 			"visible", TRUE,
 			"adjustment", adjustment,
 			"digits", 1,
-			"width_request", 2 * bst_size_width (BST_SIZE_TOOLBAR),
+			"width_request", 2 * gxk_size_width (BST_SIZE_TOOLBAR),
 			NULL);
-  bst_toolbar_append (self->toolbar, BST_TOOLBAR_EXTRA_WIDGET,
+  gxk_toolbar_append (self->toolbar, GXK_TOOLBAR_EXTRA_WIDGET,
 		      "HZoom", "Horizontal Zoom", entry);
 
   /* setup the popup menu
@@ -306,13 +302,13 @@ bst_part_dialog_finalize (GObject *object)
 
 void
 bst_part_dialog_set_proxy (BstPartDialog *self,
-			   BswProxy       part)
+			   SfiProxy       part)
 {
   g_return_if_fail (BST_IS_PART_DIALOG (self));
   if (part)
-    g_return_if_fail (BSW_IS_PART (part));
+    g_return_if_fail (BSE_IS_PART (part));
 
-  bst_dialog_sync_title_to_proxy (BST_DIALOG (self), part, "%s");
+  bst_window_sync_title_to_proxy (GXK_DIALOG (self), part, "%s");
   bst_piano_roll_set_proxy (BST_PIANO_ROLL (self->proll), part);
 }
 
@@ -369,25 +365,24 @@ part_dialog_update_tool (BstPartDialog *self)
 
 static void
 part_dialog_run_proc (GtkWidget *widget,
-		      gulong     callback_action,
+		      gulong     category_id,
 		      gpointer   popup_data)
 {
   BstPartDialog *self = BST_PART_DIALOG (widget);
-
-  GType proc_type = callback_action;
-
-  bst_procedure_exec_auto (proc_type,
-			   "part", BSE_TYPE_PART, bse_object_from_id (BST_PIANO_ROLL (self->proll)->proxy),
+  BseCategory *cat = bse_category_from_id (category_id);
+  
+  bst_procedure_exec_auto (cat->type,
+			   "part", SFI_TYPE_PROXY, BST_PIANO_ROLL (self->proll)->proxy,
 			   NULL);
 }
 
 static void
-piano_canvas_press (BstPartDialog *self,
-                    guint          button,
-                    guint          tick,
-                    gint           note,
-		    GdkEvent      *event,
-                    BstPianoRoll  *proll)
+piano_canvas_clicked (BstPartDialog *self,
+		      guint          button,
+		      guint          tick,
+		      gint           note,
+		      GdkEvent      *event,
+		      BstPianoRoll  *proll)
 {
   if (button == 3 && event)
     {
@@ -416,17 +411,21 @@ part_dialog_qnote_choice (BstPartDialog *self,
 }
 
 static void
-menu_select_tool (BstPartDialog *self,
-		  guint          tool)
+menu_select_tool (GtkWidget *owner,
+		  gulong     callback_action,
+		  gpointer   popup_data)
 {
-  bst_radio_tools_set_tool (self->rtools, tool);
+  BstPartDialog *self = BST_PART_DIALOG (owner);
+  bst_radio_tools_set_tool (self->rtools, callback_action);
 }
 
 static void
-menu_activate_tool (BstPartDialog *self,
-		    guint          tool)
+menu_activate_tool (GtkWidget *owner,
+		    gulong     callback_action,
+		    gpointer   popup_data)
 {
-  switch (tool)
+  BstPartDialog *self = BST_PART_DIALOG (owner);
+  switch (callback_action)
     {
     case ACTION_CLEAR:
       bst_piano_roll_controller_clear (self->proll_ctrl);
