@@ -26,7 +26,6 @@
 static void	bst_snet_shell_class_init	(BstSNetShellClass	*klass);
 static void	bst_snet_shell_init		(BstSNetShell		*pe);
 static void	bst_snet_shell_rebuild		(BstSuperShell		*super_shell);
-static void	bst_snet_shell_update		(BstSuperShell		*super_shell);
 static void	bst_snet_shell_operate		(BstSuperShell		*super_shell,
 						 BstOps			 sop);
 static gboolean	bst_snet_shell_can_operate	(BstSuperShell		*super_shell,
@@ -75,7 +74,6 @@ bst_snet_shell_class_init (BstSNetShellClass *class)
   super_shell_class->operate = bst_snet_shell_operate;
   super_shell_class->can_operate = bst_snet_shell_can_operate;
   super_shell_class->rebuild = bst_snet_shell_rebuild;
-  super_shell_class->update = bst_snet_shell_update;
 
   class->factories_path = "<BstSNetShell>";
 }
@@ -91,7 +89,7 @@ static void
 bst_snet_shell_rebuild (BstSuperShell *super_shell)
 {
   BstSNetShell *snet_shell = BST_SNET_SHELL (super_shell);
-  BswProxy snet = super_shell->super;
+  SfiProxy snet = super_shell->super;
   GtkWidget *notebook;
 
   g_return_if_fail (snet_shell->param_view == NULL);
@@ -114,8 +112,20 @@ bst_snet_shell_rebuild (BstSuperShell *super_shell)
 		    "signal_after::switch-page", gtk_widget_viewable_changed, NULL,
 		    NULL);
 
-  /* parameters
-   */
+  /* router */
+  if (bse_snet_supports_user_synths (snet) || BST_DVL_EXT)
+    {
+      snet_shell->snet_router = bst_snet_router_build_page (super_shell->super);
+      g_object_connect (GTK_WIDGET (snet_shell->snet_router),
+			"signal::destroy", gtk_widget_destroyed, &snet_shell->snet_router,
+			NULL);
+      gtk_notebook_append_page (GTK_NOTEBOOK (notebook), gtk_widget_get_toplevel (GTK_WIDGET (snet_shell->snet_router)),
+				gtk_widget_new (GTK_TYPE_LABEL,
+						"label", "Routing",
+						"visible", TRUE,
+						NULL));
+    }
+  /* parameters */
   snet_shell->param_view = (BstParamView*) bst_param_view_new (snet);
   g_object_set (GTK_WIDGET (snet_shell->param_view),
 		"visible", TRUE,
@@ -128,33 +138,6 @@ bst_snet_shell_rebuild (BstSuperShell *super_shell)
 					    "label", "Parameters",
 					    "visible", TRUE,
 					    NULL));
-
-  /* router
-   */
-  if (bsw_snet_supports_user_synths (snet) || BST_DVL_EXT)
-    {
-      snet_shell->snet_router = bst_snet_router_build_page (super_shell->super);
-      g_object_connect (GTK_WIDGET (snet_shell->snet_router),
-			"signal::destroy", gtk_widget_destroyed, &snet_shell->snet_router,
-			NULL);
-      gtk_notebook_append_page (GTK_NOTEBOOK (notebook), gtk_widget_get_toplevel (GTK_WIDGET (snet_shell->snet_router)),
-				gtk_widget_new (GTK_TYPE_LABEL,
-						"label", "Routing",
-						"visible", TRUE,
-						NULL));
-    }
-}
-
-static void
-bst_snet_shell_update (BstSuperShell *super_shell)
-{
-  BstSNetShell *snet_shell;
-  
-  snet_shell = BST_SNET_SHELL (super_shell);
-  
-  bst_param_view_update (snet_shell->param_view);
-  if (snet_shell->snet_router)
-    bst_snet_router_update (snet_shell->snet_router);
 }
 
 static void
