@@ -22,6 +22,8 @@
 
 #include	<bse/bsesource.h>
 
+G_BEGIN_DECLS
+
 /* --- object type macros --- */
 #define	BSE_TYPE_CONTAINER		(BSE_TYPE_ID (BseContainer))
 #define BSE_CONTAINER(object)		(G_TYPE_CHECK_INSTANCE_CAST ((object), BSE_TYPE_CONTAINER, BseContainer))
@@ -36,7 +38,7 @@
 typedef gboolean (*BseForallItemsFunc) (BseItem	 *item,
 					gpointer  data);
 typedef gboolean (*BseForallCrossFunc) (BseItem	 *owner,
-					BseItem  *ref_item,
+					BseItem  *link,
 					gpointer  data);
 struct _BseContainer
 {
@@ -59,6 +61,7 @@ struct _BseContainerClass
 					 GType			 child_type,
 					 const gchar		*uname);
   GSList*	(*context_children)	(BseContainer		*container);
+  void		(*release_children)	(BseContainer		*container);
 };
 
 
@@ -74,15 +77,14 @@ void		bse_container_remove_item	(BseContainer	*container,
 void		bse_container_forall_items	(BseContainer	*container,
 						 BseForallItemsFunc func,
 						 gpointer	 data);
-BswIterProxy*	bse_container_list_items	(BseContainer	*container);
+BseProxySeq*	bse_container_list_items	(BseContainer	*container);
 guint		bse_container_get_item_seqid	(BseContainer	*container,
 						 BseItem	*item);
 BseItem*	bse_container_get_item		(BseContainer	*container,
 						 GType  	 item_type,
 						 guint		 seq_id);
-void		bse_container_store_items	(BseContainer	*container,
-						 BseStorage	*storage,
-						 const gchar	*restore_func);
+void		bse_container_store_children	(BseContainer	*container,
+						 BseStorage	*storage);
 BseItem*	bse_container_lookup_item	(BseContainer	*container,
 						 const gchar	*uname);
 BseItem*	bse_container_retrieve_child	(BseContainer	*container,
@@ -94,26 +96,22 @@ gchar* /*fr*/	bse_container_make_upath	(BseContainer	*container,
 
 
 /* --- internal functions --- */
-void            bse_container_cross_ref         (BseContainer    *container,
+void          _bse_container_cross_link		(BseContainer    *container,
 						 BseItem         *owner,
-						 BseItem         *ref_item,
+						 BseItem         *link,
 						 BseItemUncross   uncross_func);
-void            bse_container_cross_unref       (BseContainer    *container,
+void          _bse_container_cross_unlink	(BseContainer    *container,
 						 BseItem         *owner,
-						 BseItem         *ref_item,
-						 gboolean	 notify);
-void		bse_container_uncross_item	(BseContainer    *container,
+						 BseItem         *link,
+						 BseItemUncross   uncross);
+void          _bse_container_uncross		(BseContainer    *container,
+						 BseItem         *owner,
+						 BseItem         *link);
+void	      _bse_container_uncross_descendant	(BseContainer    *container,
 						 BseItem         *item);
-void		bse_container_cross_forall	(BseContainer	*container,
-						 BseForallCrossFunc func,
-						 gpointer	 data);
 
 
 
-
-
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 #endif /* __BSE_CONTAINER_H__ */
