@@ -15,15 +15,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  */
-#include        "bse.h"
-
-#include        "gslmagic.h"
-#include        "gslcommon.h"
-#include        "gslloader.h"
-#include        "gsldatahandle.h"
-
-#include        "../PKG_config.h"
-#include        <stdio.h>
+#include "bsemain.h"
+#include "PKG_config.h"
+#include "gslmagic.h"
+#include "gslcommon.h"
+#include "gslloader.h"
+#include "gsldatahandle.h"
+#include <stdio.h>
+#include <string.h>
 
 static gint
 help (gchar *arg)
@@ -31,7 +30,7 @@ help (gchar *arg)
   fprintf (stderr, "usage: bsemagic [-{h|p|}] [files...]\n");
   fprintf (stderr, "       -p       include plugins\n");
   fprintf (stderr, "       -h       guess what ;)\n");
-
+  
   return arg != NULL;
 }
 
@@ -57,13 +56,13 @@ main (gint   argc,
   };
   static const guint n_magic_presets = sizeof (magic_presets) / sizeof (magic_presets[0]);
   guint i;
-  GslRing *magic_list = NULL;
-
+  SfiRing *magic_list = NULL;
+  
   g_thread_init (NULL);
-  bse_init (&argc, &argv, NULL);
-
+  bse_init_intern (&argc, &argv, NULL);
+  
   for (i = 0; i < n_magic_presets; i++)
-    magic_list = gsl_ring_append (magic_list,
+    magic_list = sfi_ring_append (magic_list,
 				  gsl_magic_create (magic_presets[i][0],
 						    0,
 						    0,
@@ -72,7 +71,7 @@ main (gint   argc,
   for (i = 1; i < argc; i++)
     {
       if (strcmp ("-p", argv[i]) == 0)
-	bsw_register_plugins (BSE_PATH_PLUGINS, FALSE, NULL, NULL, NULL);
+	; // FIXME: bsw_register_plugins (BSE_PATH_PLUGINS, FALSE, NULL, NULL, NULL);
       else if (strcmp ("-h", argv[i]) == 0)
 	{
 	  return help (NULL);
@@ -83,7 +82,7 @@ main (gint   argc,
 	  GslMagic *magic = gsl_magic_list_match_file (magic_list, argv[i]);
 	  guint l = strlen (argv[i]);
 	  gchar *pad;
-
+	  
 	  g_print ("%s:", argv[i]);
 	  pad = g_strnfill (MAX (40, l) - l, ' ');
 	  g_print (pad);
@@ -100,6 +99,6 @@ main (gint   argc,
 	  g_print ("\n");
 	}
     }
-
+  
   return 0;
 }
