@@ -1,5 +1,5 @@
 /* BSE - Bedevilled Sound Engine
- * Copyright (C) 1998, 1999 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998-2002 Tim Janik
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,10 +18,14 @@
 #include "bseparasite.h"
 
 #include "bsestorage.h"
+#include "bsesource.h"	// FIXME: compat include
 
+#include <string.h>
 
 /* --- defines --- */
 #define MAX_PARASITE_VALUES (1024) /* (2 << 24) */
+#define parse_or_return         bse_storage_scanner_parse_or_return
+#define peek_or_return          bse_storage_scanner_peek_or_return
 
 
 /* --- types --- */
@@ -53,51 +57,51 @@ static GQuark quark_parasite_list = 0;
 
 
 /* --- functions --- */
-static void
-store_parasite_list (BseObject  *object,
-		     BseStorage *storage,
-		     gpointer    data)
+void
+bse_parasite_store (BseObject  *object,
+		    BseStorage *storage)
 {
   ParasiteList *list;
   guint n;
   
-  list = bse_object_get_qdata (object, quark_parasite_list);
+  list = g_object_get_qdata (object, quark_parasite_list);
   if (!list)
     return;
-
+  
   for (n = 0; n < list->n_parasites; n++)
     {
       Parasite *parasite = list->parasites + n;
       gchar *name;
-
+      
       if (!parasite->n_values)
 	continue;
-
+      
       bse_storage_break (storage);
-      name = g_strdup_quoted (g_quark_to_string (parasite->quark));
+      name = g_strescape (g_quark_to_string (parasite->quark), NULL);
       bse_storage_printf (storage, "(parasite %c \"%s\"",
 			  parasite->type,
 			  name);
       switch (parasite->type)
 	{
 	  guint i;
-
+	  
 	case PARASITE_FLOAT:
 	  bse_storage_printf (storage, " %u", parasite->n_values);
 	  for (i = 0; i < parasite->n_values; i++)
 	    {
 	      gfloat *floats = parasite->data;
-
+	      
 	      if ((i + 1) % 5 == 0)
 		bse_storage_break (storage);
-	      bse_storage_printf (storage, " %f", floats[i]);
+	      bse_storage_putc (storage, ' ');
+	      bse_storage_putf (storage, floats[i]);
 	    }
 	  break;
 	default:
 	  g_warning (G_STRLOC ": unknown parasite type `%c' for \"%s\" in \"%s\"",
 		     parasite->type,
 		     name,
-		     BSE_OBJECT_NAME (object));
+		     BSE_OBJECT_UNAME (object));
 	  break;
 	}
       g_free (name);
@@ -110,7 +114,7 @@ parasite_list_free (gpointer data)
 {
   ParasiteList *list = data;
   guint i;
-
+  
   for (i = 0; i < list->n_parasites; i++)
     if (list->parasites[i].n_values)
       g_free (list->parasites[i].data);
@@ -125,15 +129,15 @@ fetch_parasite (BseObject *object,
 {
   ParasiteList *list;
   guint i;
-
-  list = bse_object_get_qdata (object, quark_parasite_list);
+  
+  list = g_object_get_qdata (object, quark_parasite_list);
   
   if (list)
     for (i = 0; i < list->n_parasites; i++)
       if (list->parasites[i].quark == quark &&
 	  list->parasites[i].type == type)
 	return list->parasites + i;
-
+  
   if (create)
     {
       ParasiteList *olist = list;
@@ -145,25 +149,20 @@ fetch_parasite (BseObject *object,
 	{
 	  if (!quark_parasite_list)
 	    quark_parasite_list = g_quark_from_static_string ("BseParasiteList");
-
+	  
 	  if (olist)
-	    bse_object_steal_qdata (object, quark_parasite_list);
-	  else
-	    bse_object_add_notifier (object,
-				     "store",
-				     store_parasite_list,
-				     NULL);
-	  bse_object_set_qdata_full (object, quark_parasite_list, list, parasite_list_free);
+	    g_object_steal_qdata (object, quark_parasite_list);
+	  g_object_set_qdata_full (object, quark_parasite_list, list, parasite_list_free);
 	}
-
+      
       list->parasites[i].quark = quark;
       list->parasites[i].type = type;
       list->parasites[i].n_values = 0;
       list->parasites[i].data = NULL;
-
+      
       return list->parasites + i;
     }
-
+  
   return NULL;
 }
 
@@ -176,10 +175,10 @@ delete_parasite (BseObject *object,
   Parasite *parasite = NULL;
   guint i;
   
-  list = bse_object_get_qdata (object, quark_parasite_list);
+  list = g_object_get_qdata (object, quark_parasite_list);
   if (!list)
     return;
-
+  
   for (i = 0; i < list->n_parasites; i++)
     if (list->parasites[i].quark == quark &&
 	list->parasites[i].type == type)
@@ -193,48 +192,49 @@ delete_parasite (BseObject *object,
   if (i < list->n_parasites)
     list->parasites[i] = list->parasites[list->n_parasites];
   else if (list->n_parasites == 0)
-    {
-      bse_object_remove_notifiers_by_func (object, store_parasite_list, NULL);
-      bse_object_set_qdata (object, quark_parasite_list, NULL);
-    }
+    g_object_set_qdata (object, quark_parasite_list, NULL);
 }
 
-static GTokenType
-parasite_parser (BseObject  *object,
-		 BseStorage *storage,
-		 gpointer    user_data)
+GTokenType
+bse_parasite_restore (BseObject  *object,
+		      BseStorage *storage)
 {
   GScanner *scanner = storage->scanner;
-  gboolean char_2_token;
   GQuark quark;
   guint type;
   guint n_values;
   gpointer data;
-
+  
   /* check identifier */
-  if (scanner->token != G_TOKEN_IDENTIFIER ||
-      !bse_string_equals ("parasite", scanner->value.v_identifier))
-    return G_TOKEN_IDENTIFIER;
+  if (g_scanner_peek_next_token (scanner) != G_TOKEN_IDENTIFIER ||
+      !bse_string_equals ("parasite", scanner->next_value.v_identifier))
+    return BSE_TOKEN_UNMATCHED;
+
+  /* eat "parasite" identifier */
+  g_scanner_get_next_token (scanner);
+
+  /* ignore these for compat reasons (FIXME: remove post 0.5.0) */
+  if (g_scanner_peek_next_token (scanner) == '#')
+    g_scanner_get_next_token (scanner);
+  if (g_scanner_peek_next_token (scanner) == '\\')
+    g_scanner_get_next_token (scanner);
 
   /* parse parasite type */
-  char_2_token = scanner->config->char_2_token;
-  scanner->config->char_2_token = FALSE;
   g_scanner_get_next_token (scanner);
-  scanner->config->char_2_token = char_2_token;
-  if (scanner->token != G_TOKEN_CHAR)
+  if (!(scanner->token >= 'a' && scanner->token <= 'z'))
     return G_TOKEN_CHAR;
-  type = scanner->value.v_char;
+  type = scanner->token;
 
   /* parse parasite name */
   if (g_scanner_get_next_token (scanner) != G_TOKEN_STRING)
     return G_TOKEN_STRING;
   quark = g_quark_from_string (scanner->value.v_string);
-
+  
   switch (type)
     {
       guint i;
       gfloat *floats;
-
+      
     case PARASITE_FLOAT:
       if (g_scanner_get_next_token (scanner) != G_TOKEN_INT)
 	return G_TOKEN_INT;
@@ -245,18 +245,23 @@ parasite_parser (BseObject  *object,
       for (i = 0; i < n_values; i++)
 	{
 	  gboolean negate = FALSE;
+	  gfloat vfloat;
 
 	  if (g_scanner_get_next_token (scanner) == '-')
 	    {
 	      g_scanner_get_next_token (scanner);
 	      negate = TRUE;
 	    }
-	  if (scanner->token != G_TOKEN_FLOAT)
+	  if (scanner->token == G_TOKEN_INT)
+	    vfloat = scanner->config->store_int64 ? scanner->value.v_int64 : scanner->value.v_int;
+	  else if (scanner->token == G_TOKEN_FLOAT)
+	    vfloat = scanner->value.v_float;
+	  else
 	    {
 	      g_free (floats);
 	      return G_TOKEN_FLOAT;
 	    }
-	  floats[i] = negate ? - scanner->value.v_float : scanner->value.v_float;
+	  floats[i] = negate ? - vfloat : vfloat;
 	}
       data = floats;
       break;
@@ -267,31 +272,37 @@ parasite_parser (BseObject  *object,
 				    type,
 				    g_quark_to_string (quark));
     }
-
+  
   if (g_scanner_peek_next_token (scanner) == ')')
     {
-      Parasite *parasite = fetch_parasite (object, quark, type, TRUE);
-
-      if (parasite->n_values)
-	g_free (parasite->data);
-      parasite->n_values = n_values;
-      parasite->data = data;
+      if (n_values == 2 && BSE_IS_SOURCE (object) &&
+	  quark == g_quark_from_string ("BstRouterCoords"))  // FIXME: compat code, remove
+	{
+	  gfloat *floats = data;
+	  g_object_set (object,
+			"pos_x", -floats[0] / 100.0,
+			"pos_y", floats[1] / 100.0,
+			NULL);
+	  bse_storage_warn (storage, "fixing up parasite to module position: (%f,%f)", -floats[0] / 100.0, floats[1] / 100.0);
+	  g_free (data);
+	}
+      else
+	{
+	  Parasite *parasite = fetch_parasite (object, quark, type, TRUE);
+	  
+	  if (parasite->n_values)
+	    g_free (parasite->data);
+	  parasite->n_values = n_values;
+	  parasite->data = data;
+	}
     }
   else if (n_values)
     g_free (data);
-
+  
   /* read closing brace */
   return g_scanner_get_next_token (scanner) == ')' ? G_TOKEN_NONE : ')';
 }
 
-void
-bse_parasite_install_parsers (BseObjectClass *oclass)
-{
-  g_return_if_fail (BSE_IS_OBJECT_CLASS (oclass));
-
-  bse_object_class_add_parser (oclass, "parasite", parasite_parser, NULL);
-}
-
 void
 bse_parasite_set_floats (BseObject   *object,
 			 const gchar *name,
@@ -303,7 +314,7 @@ bse_parasite_set_floats (BseObject   *object,
   g_return_if_fail (n_values < MAX_PARASITE_VALUES);
   if (n_values)
     g_return_if_fail (float_values != NULL);
-
+  
   if (!n_values)
     delete_parasite (object, g_quark_try_string (name), PARASITE_FLOAT);
   else
@@ -312,7 +323,7 @@ bse_parasite_set_floats (BseObject   *object,
 					   g_quark_from_string (name),
 					   PARASITE_FLOAT,
 					   TRUE);
-
+      
       if (parasite->n_values != n_values)
 	{
 	  if (parasite->n_values)
@@ -324,32 +335,22 @@ bse_parasite_set_floats (BseObject   *object,
     }
 }
 
-guint
+SfiFBlock*
 bse_parasite_get_floats (BseObject   *object,
-			 const gchar *name,
-			 guint        max_n_values,
-			 gfloat      *float_values)
+			 const gchar *name)
 {
   Parasite *parasite;
-  guint i, n;
-
+  SfiFBlock *fblock;
+  
   g_return_val_if_fail (BSE_IS_OBJECT (object), 0);
   g_return_val_if_fail (name != NULL, 0);
-  if (max_n_values)
-    g_return_val_if_fail (float_values != NULL, 0);
-
+  
   parasite = fetch_parasite (object,
 			     g_quark_try_string (name),
 			     PARASITE_FLOAT,
 			     FALSE);
-  n = parasite ? parasite->n_values : 0;
-  i = MIN (n, max_n_values);
-  if (i)
-    memcpy (float_values, parasite->data, i * sizeof (gfloat));
-  max_n_values -= i;
-  float_values += i;
-  while (max_n_values--)
-    *(float_values++) = 0.0;
-
-  return n;
+  fblock = sfi_fblock_new ();
+  if (parasite)
+    sfi_fblock_append (fblock, parasite->n_values, parasite->data);
+  return fblock;
 }
