@@ -1,5 +1,5 @@
 /* BSE - Bedevilled Sound Engine
- * Copyright (C) 1998, 1999 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998-2002 Tim Janik
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,19 +29,16 @@ extern "C" {
 
 
 /* --- prototypes --- */
-void	bse_parasite_set_floats		(BseObject      *object,
+void	   bse_parasite_set_floats	(BseObject      *object,
 					 const gchar	*name,
 					 guint		 n_values,
 					 gfloat		*float_values);
-guint	bse_parasite_get_floats		(BseObject      *object,
-					 const gchar	*name,
-					 guint		 max_n_values,
-					 gfloat		*float_values);
-
-
-/* --- internal --- */
-void	bse_parasite_install_parsers	(BseObjectClass	*oclass);
-
+SfiFBlock* bse_parasite_get_floats	(BseObject      *object,
+					 const gchar	*name);
+void	   bse_parasite_store		(BseObject	*object,
+					 BseStorage	*storage);
+GTokenType bse_parasite_restore		(BseObject	*object,
+					 BseStorage	*storage);
 
 
 #ifdef __cplusplus
