@@ -17,7 +17,7 @@
  */
 #include	"bsepcmdevice-oss.h"
 
-#include	"../PKG_config.h"
+#include	"PKG_config.h"
 
 #include	"gsldatautils.h"
 
@@ -30,6 +30,7 @@ BSE_DUMMY_TYPE (BsePcmDeviceOSS);
 #include	<sys/types.h>
 #include	<sys/time.h>
 #include	<unistd.h>
+#include	<string.h>
 #include	<errno.h>
 #include	<fcntl.h>
 
@@ -41,6 +42,8 @@ BSE_DUMMY_TYPE (BsePcmDeviceOSS);
 #error	unsupported byte order in G_BYTE_ORDER
 #endif
 
+#define DEBUG   sfi_debug_keyfunc ("pcm")
+
 
 /* --- OSS PCM handle --- */
 typedef struct
@@ -58,7 +61,7 @@ typedef struct
 /* --- prototypes --- */
 static void	    bse_pcm_device_oss_class_init	(BsePcmDeviceOSSClass	*class);
 static void	    bse_pcm_device_oss_init		(BsePcmDeviceOSS	*pcm_device_oss);
-static void	    bse_pcm_device_oss_destroy		(BseObject		*object);
+static void	    bse_pcm_device_oss_finalize		(GObject		*object);
 static BseErrorType bse_pcm_device_oss_open		(BsePcmDevice		*pdev);
 static BseErrorType oss_device_setup			(OSSHandle		*oss);
 static void	    oss_device_retrigger		(OSSHandle		*oss);
@@ -106,13 +109,13 @@ BSE_BUILTIN_TYPE (BsePcmDeviceOSS)
 static void
 bse_pcm_device_oss_class_init (BsePcmDeviceOSSClass *class)
 {
-  BseObjectClass *object_class = BSE_OBJECT_CLASS (class);
+  GObjectClass *gobject_class = G_OBJECT_CLASS (class);
   BsePcmDeviceClass *pcm_device_class = BSE_PCM_DEVICE_CLASS (class);
-
+  
   parent_class = g_type_class_peek_parent (class);
-
-  object_class->destroy = bse_pcm_device_oss_destroy;
-
+  
+  gobject_class->finalize = bse_pcm_device_oss_finalize;
+  
   pcm_device_class->open = bse_pcm_device_oss_open;
   pcm_device_class->suspend = bse_pcm_device_oss_close;
   pcm_device_class->driver_rating = BSE_RATING_DEFAULT;
@@ -130,7 +133,7 @@ bse_pcm_device_oss_open (BsePcmDevice *pdev)
   OSSHandle *oss = g_new0 (OSSHandle, 1);
   BsePcmHandle *handle = &oss->handle;
   BseErrorType error = BSE_ERROR_NONE;
-
+  
   /* setup request */
   handle->writable = TRUE;
   handle->readable = TRUE;
@@ -144,29 +147,34 @@ bse_pcm_device_oss_open (BsePcmDevice *pdev)
   oss->frag_size = 512;
   oss->bytes_per_value = 2;
   oss->frag_buf = NULL;
-
+  
   /* try open */
   if (!error)
     {
-      gint omode = 0;
-      gint fd;
-      
-      omode = (handle->readable && handle->writable ? O_RDWR
-	       : handle->readable ? O_RDONLY
-	       : handle->writable ? O_WRONLY : 0);
-      
-      /* need to open explicitely non-blocking or we'll have to wait untill someone else closes the device */
-      fd = open (BSE_PCM_DEVICE_OSS (pdev)->device_name, omode | O_NONBLOCK, 0);
+      gint omode, fd;
+
+      g_assert (handle->readable && handle->writable);
+      omode = O_RDWR | O_NONBLOCK;
+      /* need to open explicitely non-blocking or we'll have to wait until someone else closes the device */
+      fd = open (BSE_PCM_DEVICE_OSS (pdev)->device_name, omode, 0);
+      if (fd < 0)
+	{
+	  /* retry without recording */
+	  handle->readable = FALSE;
+	  g_assert (handle->readable == FALSE && handle->writable);
+	  omode = O_WRONLY | O_NONBLOCK;
+	  fd = open (BSE_PCM_DEVICE_OSS (pdev)->device_name, omode, 0);
+	}
       if (fd >= 0)
 	oss->fd = fd;
       else
-	error = bse_error_from_errno (errno, BSE_ERROR_OPEN_FAILED);
+	error = bse_error_from_errno (errno, BSE_ERROR_FILE_OPEN_FAILED);
     }
-
+  
   /* try setup */
   if (!error)
     error = oss_device_setup (oss);
-
+  
   /* setup pdev or shutdown */
   if (!error)
     {
@@ -188,32 +196,32 @@ bse_pcm_device_oss_open (BsePcmDevice *pdev)
     }
   else
     {
-      if (oss->fd < 0)
+      if (oss->fd >= 0)
 	close (oss->fd);
       g_free (oss->frag_buf);
       g_free (oss);
     }
-
+  
   return error;
 }
 
 static void
-bse_pcm_device_oss_destroy (BseObject *object)
+bse_pcm_device_oss_finalize (GObject *object)
 {
   BsePcmDeviceOSS *pdev_oss = BSE_PCM_DEVICE_OSS (object);
-
+  
   g_free (pdev_oss->device_name);
   pdev_oss->device_name = NULL;
   
-  /* chain parent class' destroy handler */
-  BSE_OBJECT_CLASS (parent_class)->destroy (object);
+  /* chain parent class' handler */
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static void
 bse_pcm_device_oss_close (BsePcmDevice *pdev)
 {
   OSSHandle *oss = (OSSHandle*) pdev->handle;
-
+  
   pdev->handle = NULL;
   (void) ioctl (oss->fd, SNDCTL_DSP_RESET);
   (void) close (oss->fd);
@@ -228,12 +236,12 @@ oss_device_setup (OSSHandle *oss)
   gint fd = oss->fd;
   glong d_long;
   gint d_int;
-
+  
   d_long = fcntl (fd, F_GETFL);
   d_long &= ~O_NONBLOCK;
   if (fcntl (fd, F_SETFL, d_long))
     return BSE_ERROR_DEVICE_ASYNC;
-
+  
   d_int = 0;
   if (ioctl (fd, SNDCTL_DSP_GETFMTS, &d_int) < 0)
     return BSE_ERROR_DEVICE_GET_CAPS;
@@ -244,7 +252,7 @@ oss_device_setup (OSSHandle *oss)
       d_int != AFMT_S16_HE)
     return BSE_ERROR_DEVICE_SET_CAPS;
   oss->bytes_per_value = 2;
-
+  
   d_int = handle->n_channels - 1;
   if (ioctl (fd, SNDCTL_DSP_STEREO, &d_int) < 0)
     return BSE_ERROR_DEVICE_SET_CAPS;
@@ -254,7 +262,7 @@ oss_device_setup (OSSHandle *oss)
   if (ioctl (fd, SNDCTL_DSP_SPEED, &d_int) < 0)
     return BSE_ERROR_DEVICE_SET_CAPS;
   handle->mix_freq = d_int;
-
+  
   /* Note: fragment = n_fragments << 16;
    *       fragment |= g_bit_storage (fragment_size - 1);
    */
@@ -271,11 +279,11 @@ oss_device_setup (OSSHandle *oss)
       (d_int & 1))
     return BSE_ERROR_DEVICE_GET_CAPS;
   /* handle->block_size = d_int; */
-
+  
   if (handle->writable)
     {
       audio_buf_info info = { 0, };
-
+      
       if (ioctl (fd, SNDCTL_DSP_GETOSPACE, &info) < 0)
 	return BSE_ERROR_DEVICE_GET_CAPS;
       oss->frag_size = info.fragsize;
@@ -284,23 +292,22 @@ oss_device_setup (OSSHandle *oss)
   else if (handle->readable)
     {
       audio_buf_info info = { 0, };
-
+      
       if (ioctl (fd, SNDCTL_DSP_GETISPACE, &info) < 0)
 	return BSE_ERROR_DEVICE_GET_CAPS;
       oss->frag_size = info.fragsize;
       oss->n_frags = info.fragstotal;
     }
   
-  BSE_IF_DEBUG (PCM)
-    g_message ("OSS-SETUP: w=%d r=%d n_channels=%d sample_freq=%.0f fsize=%u nfrags=%u bufsz=%u\n",
-	       handle->writable,
-	       handle->readable,
-	       handle->n_channels,
-	       handle->mix_freq,
-	       oss->frag_size,
-	       oss->n_frags,
-	       oss->n_frags * oss->frag_size);
-
+  DEBUG ("OSS-SETUP: w=%d r=%d n_channels=%d sample_freq=%.0f fsize=%u nfrags=%u bufsz=%u\n",
+	 handle->writable,
+	 handle->readable,
+	 handle->n_channels,
+	 handle->mix_freq,
+	 oss->frag_size,
+	 oss->n_frags,
+	 oss->n_frags * oss->frag_size);
+  
   return BSE_ERROR_NONE;
 }
 
@@ -308,7 +315,7 @@ static void
 oss_device_retrigger (OSSHandle *oss)
 {
   gint d_int;
-
+  
   /* it should be enough to select() on the fd to trigger
    * capture/playback, but with some new OSS drivers
    * (clones) this is not the case anymore, so we also
@@ -321,7 +328,7 @@ oss_device_retrigger (OSSHandle *oss)
   if (oss->handle.writable)
     d_int |= PCM_ENABLE_OUTPUT;
   (void) ioctl (oss->fd, SNDCTL_DSP_SETTRIGGER, &d_int);
-
+  
   /* Jaroslav Kysela <perex@jcu.cz>:
    *  Important sequence:
    *     1) turn on capture
@@ -343,7 +350,7 @@ oss_device_retrigger (OSSHandle *oss)
       struct timeval tv = { 0, 0, };
       fd_set in_fds;
       fd_set out_fds;
-
+      
       FD_ZERO (&in_fds);
       FD_ZERO (&out_fds);
       FD_SET (oss->fd, &in_fds);
@@ -359,7 +366,7 @@ oss_device_status (BsePcmHandle *handle,
   OSSHandle *oss = (OSSHandle*) handle;
   gint fd = oss->fd;
   audio_buf_info info;
-
+  
   if (handle->readable)
     {
       memset (&info, 0, sizeof (info));
@@ -370,14 +377,13 @@ oss_device_status (BsePcmHandle *handle,
       status->n_capture_values_available = info.bytes / oss->bytes_per_value;
       /* OSS-bug fix, at least for es1371 in 2.3.34 */
       status->n_capture_values_available = MIN (status->total_capture_values, status->n_capture_values_available);
-      BSE_IF_DEBUG (PCM)
-	g_message ("OSS-ISPACE: left=%5d/%d frags: total=%d size=%d count=%d bytes=%d\n",
-		   status->n_capture_values_available,
-		   status->total_capture_values,
-		   info.fragstotal,
-		   info.fragsize,
-		   info.fragments,
-		   info.bytes);
+      DEBUG ("OSS-ISPACE: left=%5d/%d frags: total=%d size=%d count=%d bytes=%d\n",
+	     status->n_capture_values_available,
+	     status->total_capture_values,
+	     info.fragstotal,
+	     info.fragsize,
+	     info.fragments,
+	     info.bytes);
     }
   else
     {
@@ -394,14 +400,13 @@ oss_device_status (BsePcmHandle *handle,
       status->n_playback_values_available = info.bytes / oss->bytes_per_value;
       /* OSS-bug fix, at least for es1371 in 2.3.34 */
       status->n_playback_values_available = MIN (status->total_playback_values, status->n_playback_values_available);
-      BSE_IF_DEBUG (PCM)
-	g_message ("OSS-OSPACE: left=%5d/%d frags: total=%d size=%d count=%d bytes=%d\n",
-		   status->n_playback_values_available,
-		   status->total_playback_values,
-		   info.fragstotal,
-		   info.fragsize,
-		   info.fragments,
-		   info.bytes);
+      DEBUG ("OSS-OSPACE: left=%5d/%d frags: total=%d size=%d count=%d bytes=%d\n",
+	     status->n_playback_values_available,
+	     status->total_playback_values,
+	     info.fragstotal,
+	     info.fragsize,
+	     info.fragments,
+	     info.bytes);
     }
   else
     {
@@ -423,13 +428,13 @@ oss_device_read (BsePcmHandle *handle,
   gsize n_left = n_values;
   
   g_return_val_if_fail (oss->bytes_per_value == 2, 0);
-
+  
   do
     {
       gsize n = MIN (buf_size, n_left << 1);
       gint16 *b, *s = buf;
       gssize l;
-
+      
       do
 	l = read (fd, buf, n);
       while (l < 0 && errno == EINTR); /* don't mind signals */
@@ -444,7 +449,7 @@ oss_device_read (BsePcmHandle *handle,
       n_left -= l;
     }
   while (n_left);
-
+  
   return n_values;
 }
 
@@ -458,14 +463,14 @@ oss_device_write (BsePcmHandle *handle,
   gsize buf_size = FRAG_BUF_SIZE (oss);
   gpointer buf = oss->frag_buf;
   const gfloat *s = values;
-
+  
   g_return_if_fail (oss->bytes_per_value == 2);
-
+  
   do
     {
       gsize n = MIN (buf_size, n_values << 1);
       gssize l;
-
+      
       gsl_conv_from_float_clip (GSL_WAVE_FORMAT_SIGNED_16,
 				G_BYTE_ORDER,
 				s,
