@@ -1,5 +1,5 @@
 /* BSE - Bedevilled Sound Engine
- * Copyright (C) 1998, 1999 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998-2002 Tim Janik
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,22 +28,113 @@
 #include	<unistd.h>
 
 
+#define	BUILTIN_USE_COUNT	(1)
+#define DEBUG   sfi_debug_keyfunc ("plugins")
+
 
 /* --- prototypes --- */
-static const gchar* bse_plugin_register_types	(BsePlugin      *plugin,
-						 gconstpointer   array_p,
-						 BseExportType   exp_type);
-
+static void	    bse_plugin_init		(BsePlugin	  *plugin);
+static void	    bse_plugin_class_init	(BsePluginClass	  *class);
+static const gchar* bse_plugin_register_exports	(BsePlugin        *plugin,
+						 gconstpointer     array_p,
+						 BseExportType     exp_type);
+static void	    bse_plugin_complete_info	(GTypePlugin	  *gplugin,
+						 GType       	   type,
+						 GTypeInfo 	  *type_info,
+						 GTypeValueTable  *value_vtable);
+static void	    bse_plugin_use		(GTypePlugin	  *gplugin);
+static void	    bse_plugin_unuse		(GTypePlugin	  *gplugin);
+static void	    type_plugin_iface_init	(GTypePluginClass *iface);
 
 /* --- variables --- */
-gconstpointer BSE_EXPORT_IMPL_S (Procedure) = NULL;
-gconstpointer BSE_EXPORT_IMPL_S (Object) = NULL;
-BSE_EXPORT_IMPL_C = NULL;
+gconstpointer BSE_EXPORT_IMPL_S (Procedure) = NULL;	/* placeholder for proc-less builtins */
+gconstpointer BSE_EXPORT_IMPL_S (Object) = NULL;	/* placeholder for obj-less builtins */
+BSE_EXPORT_IMPL_T = NULL;				/* bse_plugin_builtin_init() declaration */
 static GSList    *bse_plugins = NULL;
 
+
 /* --- functions --- */
+BSE_BUILTIN_TYPE (BsePlugin)
+{
+  static const GTypeInfo plugin_info = {
+    sizeof (BsePluginClass),
+    NULL,           /* base_init */
+    NULL,           /* base_finalize */
+    (GClassInitFunc) bse_plugin_class_init,
+    NULL,           /* class_finalize */
+    NULL,           /* class_data */
+    sizeof (BsePlugin),
+    0,		/* n_preallocs */
+    (GInstanceInitFunc) bse_plugin_init,
+  };
+  static const GInterfaceInfo iface_info = {
+    (GInterfaceInitFunc) type_plugin_iface_init,
+    NULL,		/* interface_finalize */
+    NULL,		/* interface_data */
+  };
+  GType plugin_type;
+  
+  plugin_type = g_type_register_static (G_TYPE_OBJECT, "BsePlugin", &plugin_info, 0);
+  
+  g_type_add_interface_static (plugin_type, G_TYPE_TYPE_PLUGIN, &iface_info);
+  
+  return plugin_type;
+}
+
+static void
+type_plugin_iface_init (GTypePluginClass *iface)
+{
+  iface->use_plugin = bse_plugin_use;
+  iface->unuse_plugin = bse_plugin_unuse;
+  iface->complete_type_info = bse_plugin_complete_info;
+}
+
+static void
+bse_plugin_dispose (GObject *object)
+{
+  BsePlugin *plugin = BSE_PLUGIN (object);
+  
+  g_warning (G_STRLOC ": dispose should never happen for static plugins");
+  
+  g_object_ref (object);
+  
+  /* chain parent class handler */
+  G_OBJECT_CLASS (g_type_class_peek_parent (BSE_PLUGIN_GET_CLASS (plugin)))->dispose (object);
+}
+
+static void
+bse_plugin_class_init (BsePluginClass *class)
+{
+  GObjectClass *gobject_class = G_OBJECT_CLASS (class);
+  
+  gobject_class->dispose = bse_plugin_dispose;
+}
+
+static void
+bse_plugin_init (BsePlugin *plugin)
+{
+  plugin->name = NULL;
+  plugin->fname = NULL;
+  plugin->gmodule = NULL;
+  plugin->use_count = 0;
+  plugin->exports_procedures = FALSE;
+  plugin->exports_objects = FALSE;
+  plugin->exports_enums = FALSE;
+  plugin->exports_file_handlers = FALSE;
+  plugin->n_proc_types = 0;
+  plugin->proc_types = NULL;
+  plugin->n_object_types = 0;
+  plugin->object_types = NULL;
+  plugin->n_enum_types = 0;
+  plugin->enum_types = NULL;
+  plugin->e_enums = NULL;
+  plugin->e_objects = NULL;
+  plugin->e_procs = NULL;
+  plugin->e_file_handlers = NULL;
+}
+
 void
-bse_plugin_init (void)
+bse_plugins_init (void)
 {
   /* this function is called upon initialization of BSE
    */
@@ -62,21 +153,22 @@ bse_plugin_init (void)
       
       /* initialize builtin plugins
        */
-      bse_plugin_builtin_init = bse_plugin_register_types;
+      bse_plugin_builtin_init = bse_plugin_register_exports;
       for (i = 0; i < n_builtin_inits; i++)
 	{
 	  const gchar *name, *error;
-	  BsePlugin *plugin = g_new0 (BsePlugin, 1); /* FIXME: alloc only */
-
+	  BsePlugin *plugin = g_object_new (BSE_TYPE_PLUGIN, NULL);
+	  
+	  g_free (plugin->name);
+	  plugin->name = "BSE-builtin";	/* has to get reinitialized */
 	  bse_plugins = g_slist_prepend (bse_plugins, plugin);
-	  plugin->name = "BSE-Builtin"; /* this is usually reset */
-	  plugin->fname = NULL;
-	  plugin->gmodule = NULL;
-	  plugin->module_refs = 42;
-
-	  BSE_IF_DEBUG (PLUGINS)
-	    g_message ("register-builtin-plugin \"%s\"", plugin->name);
-
+	  plugin->use_count = BUILTIN_USE_COUNT;
+	  while (plugin->use_count--)
+	    g_object_ref (G_OBJECT (plugin));
+	  plugin->use_count = BUILTIN_USE_COUNT;
+	  
+	  DEBUG ("register-builtin-plugin \"%s\"", plugin->name);
+	  
 	  name = plugin->name;
 	  error = builtin_inits[i] (plugin);
 	  if (error)
@@ -84,16 +176,16 @@ bse_plugin_init (void)
 	  if (name == plugin->name)
 	    g_warning ("builtin plugin initializer (%p) doesn't assign plugin name",
 		       builtin_inits[i]);
-
+	  
 	}
       bse_plugin_builtin_init = NULL;
     }
 }
 
 static const gchar*
-bse_plugin_register_types (BsePlugin    *plugin,
-			   gconstpointer array_p,
-			   BseExportType exp_type)
+bse_plugin_register_exports (BsePlugin    *plugin,
+			     gconstpointer array_p,
+			     BseExportType exp_type)
 {
   switch (plugin && array_p ? exp_type : 0)
     {
@@ -108,16 +200,15 @@ bse_plugin_register_types (BsePlugin    *plugin,
 	      pspec->init &&
 	      pspec->exec)
 	    {
-	      BseType type;
+	      GType   type;
 	      guint on = plugin->n_proc_types;
 	      const gchar *error;
 	      
-	      BSE_IF_DEBUG (PLUGINS)
-		g_message ("register-procedure: \"%s\"", pspec->name);
+	      DEBUG ("register-procedure: \"%s\"", pspec->name);
 	      
-	      type = bse_type_from_name (pspec->name);
+	      type = g_type_from_name (pspec->name);
 	      if (type)
-		return "Attempt to register known symbol";
+		return "Attempt to register known procedure";
 	      
 	      error = bse_procedure_type_register (pspec->name,
 						   pspec->blurb,
@@ -130,7 +221,7 @@ bse_plugin_register_types (BsePlugin    *plugin,
 		bse_categories_register_icon (pspec->category, type, &pspec->pixdata);
 	      
 	      plugin->n_proc_types++;
-	      plugin->proc_types = g_renew (BseType,
+	      plugin->proc_types = g_renew (GType,
 					    plugin->proc_types,
 					    plugin->n_proc_types);
 	      plugin->proc_types[on] = type;
@@ -148,16 +239,15 @@ bse_plugin_register_types (BsePlugin    *plugin,
 	  if (ospec->name &&
 	      ospec->object_info)
 	    {
-	      BseType type;
+	      GType   type;
 	      guint on = plugin->n_object_types;
 	      const gchar *error;
 	      
-	      BSE_IF_DEBUG (PLUGINS)
-		g_message ("register-object: \"%s\"", ospec->name);
+	      DEBUG ("register-object: \"%s\"", ospec->name);
 	      
-	      type = bse_type_from_name (ospec->name);
+	      type = g_type_from_name (ospec->name);
 	      if (type)
-		return "Attempt to register known symbol";
+		return "Attempt to register known object";
 	      
 	      error = bse_object_type_register (ospec->name,
 						ospec->parent_type,
@@ -172,7 +262,7 @@ bse_plugin_register_types (BsePlugin    *plugin,
 		bse_categories_register_icon (ospec->category, type, &ospec->pixdata);
 	      
 	      plugin->n_object_types++;
-	      plugin->object_types = g_renew (BseType,
+	      plugin->object_types = g_renew (GType,
 					      plugin->object_types,
 					      plugin->n_object_types);
 	      plugin->object_types[on] = type;
@@ -191,26 +281,27 @@ bse_plugin_register_types (BsePlugin    *plugin,
 	      espec->parent_type &&
 	      espec->values)
 	    {
-	      BseType type;
+	      GType   type;
 	      guint on = plugin->n_enum_types;
-	      const gchar *error;
 	      
-	      BSE_IF_DEBUG (PLUGINS)
-		g_message ("register-enum: \"%s\"", espec->name);
+	      DEBUG ("register-enum: \"%s\"", espec->name);
 	      
-	      type = bse_type_from_name (espec->name);
+	      type = g_type_from_name (espec->name);
 	      if (type)
-		return "Attempt to register known symbol";
-	      
-	      error = bse_enum_type_register (espec->name,
-					      espec->parent_type,
-					      plugin,
-					      &type);
-	      if (error)
-		return error;
+		return "Attempt to register known type";
+	      if (!G_TYPE_IS_ENUM (espec->parent_type) && !G_TYPE_IS_FLAGS (espec->parent_type))
+		return "Invalid enum/flags registration attempt";
 	      
+	      type = bse_type_register_dynamic (espec->parent_type,
+						espec->name, NULL,
+						plugin);
+	      /* FIXME: can't register dynamic type transforms with glib-2.2.1
+	       * g_value_register_transform_func (SFI_TYPE_CHOICE, type, sfi_value_choice2enum_simple);
+	       * g_value_register_transform_func (type, SFI_TYPE_CHOICE, sfi_value_enum2choice);
+	       */
+
 	      plugin->n_enum_types++;
-	      plugin->enum_types = g_renew (BseType,
+	      plugin->enum_types = g_renew (GType,
 					    plugin->enum_types,
 					    plugin->n_enum_types);
 	      plugin->enum_types[on] = type;
@@ -221,6 +312,9 @@ bse_plugin_register_types (BsePlugin    *plugin,
 	}
       plugin->e_enums = array_p;
       return NULL;
+      
+    default:
+      break;
     }
   
   return bse_error_blurb (BSE_ERROR_INTERNAL);
@@ -229,70 +323,24 @@ bse_plugin_register_types (BsePlugin    *plugin,
 static const gchar*
 bse_plugin_reinit_type_ids (BsePlugin *plugin)
 {
-  const BseExportProcedure *pspecs = plugin->e_procs;
-  const BseExportObject *ospecs = plugin->e_objects;
   const BseExportEnum *especs = plugin->e_enums;
+  const BseExportObject *ospecs = plugin->e_objects;
+  const BseExportProcedure *pspecs = plugin->e_procs;
   gchar *error = NULL;
   
-  /* procedure types
-   */
-  if (pspecs)
-    {
-      const BseExportProcedure *pspec;
-      BseType *exp_types = plugin->proc_types;
-      BseType *exp_last = plugin->proc_types + plugin->n_proc_types;
-      
-      for (pspec = pspecs; pspec->type_p && exp_types < exp_last; pspec++)
-	if (pspec->name &&
-	    pspec->init &&
-	    pspec->exec &&
-	    bse_type_from_name (pspec->name) == *exp_types)
-	  {
-	    if (*pspec->type_p)
-	      g_warning ("while reinitializing \"%s\", type id for `%s' already assigned?",
-			 plugin->name,
-			 pspec->name);
-	    *pspec->type_p = *(exp_types++);
-	  }
-      if (exp_types < exp_last)
-	error = "Unable to rematch procedure types from previous initialization";
-    }
-
-  /* object types
-   */
-  if (ospecs)
-    {
-      const BseExportObject *ospec;
-      BseType *exp_types = plugin->object_types;
-      BseType *exp_last = plugin->object_types + plugin->n_object_types;
-      
-      for (ospec = ospecs; ospec->type_p && exp_types < exp_last; ospec++)
-	if (ospec->object_info &&
-	    bse_type_from_name (ospec->name) == *exp_types)
-	  {
-	    if (*ospec->type_p)
-	      g_warning ("while reinitializing \"%s\", type id for `%s' already assigned?",
-			 plugin->name,
-			 ospec->name);
-	    *ospec->type_p = *(exp_types++);
-	  }
-      if (exp_types < exp_last)
-	error = "Unable to rematch object types from previous initialization";
-    }
-
   /* enum types
    */
   if (especs)
     {
       const BseExportEnum *espec;
-      BseType *exp_types = plugin->enum_types;
-      BseType *exp_last = plugin->enum_types + plugin->n_enum_types;
+      GType *exp_types = plugin->enum_types;
+      GType *exp_last = plugin->enum_types + plugin->n_enum_types;
       
       for (espec = especs; espec->type_p && exp_types < exp_last; espec++)
 	if (espec->name &&
 	    espec->parent_type &&
 	    espec->values &&
-	    bse_type_from_name (espec->name) == *exp_types)
+	    g_type_from_name (espec->name) == *exp_types)
 	  {
 	    if (*espec->type_p)
 	      g_warning ("while reinitializing \"%s\", type id for `%s' already assigned?",
@@ -303,59 +351,82 @@ bse_plugin_reinit_type_ids (BsePlugin *plugin)
       if (exp_types < exp_last)
 	error = "Unable to rematch enum types from previous initialization";
     }
-
+  
+  /* object types
+   */
+  if (ospecs)
+    {
+      const BseExportObject *ospec;
+      GType   *exp_types = plugin->object_types;
+      GType   *exp_last = plugin->object_types + plugin->n_object_types;
+      
+      for (ospec = ospecs; ospec->type_p && exp_types < exp_last; ospec++)
+	if (ospec->object_info &&
+	    g_type_from_name (ospec->name) == *exp_types)
+	  {
+	    if (*ospec->type_p)
+	      g_warning ("while reinitializing \"%s\", type id for `%s' already assigned?",
+			 plugin->name,
+			 ospec->name);
+	    *ospec->type_p = *(exp_types++);
+	  }
+      if (exp_types < exp_last)
+	error = "Unable to rematch object types from previous initialization";
+    }
+  
+  /* procedure types
+   */
+  if (pspecs)
+    {
+      const BseExportProcedure *pspec;
+      GType *exp_types = plugin->proc_types;
+      GType *exp_last = plugin->proc_types + plugin->n_proc_types;
+      
+      for (pspec = pspecs; pspec->type_p && exp_types < exp_last; pspec++)
+	if (pspec->name &&
+	    pspec->init &&
+	    pspec->exec &&
+	    g_type_from_name (pspec->name) == *exp_types)
+	  {
+	    if (*pspec->type_p)
+	      g_warning ("while reinitializing \"%s\", type id for `%s' already assigned?",
+			 plugin->name,
+			 pspec->name);
+	    *pspec->type_p = *(exp_types++);
+	  }
+      if (exp_types < exp_last)
+	error = "Unable to rematch procedure types from previous initialization";
+    }
+  
+  /* nothing to do for file handler types (BseExportFileHandler) */
+  
   return error;
 }
 
-void
-bse_plugin_ref (BsePlugin *plugin)
+static void
+bse_plugin_use (GTypePlugin *gplugin)
 {
+  BsePlugin *plugin = BSE_PLUGIN (gplugin);
   gboolean need_reinit = FALSE;
-
+  
   g_return_if_fail (plugin != NULL);
   
-  if (!plugin->module_refs)
+  g_object_ref (G_OBJECT (plugin));
+  if (!plugin->use_count)
     {
       const gchar *error = NULL;
       
-      BSE_IF_DEBUG (PLUGINS)
-	g_message ("reloading-plugin \"%s\"", plugin->name);
+      plugin->use_count++;
+      
+      DEBUG ("reloading-plugin \"%s\" (\"%s\")", plugin->name, plugin->fname ? plugin->fname : "???NULL???");
       
       plugin->gmodule = g_module_open (plugin->fname, 0);
       if (!plugin->gmodule)
 	error = g_module_error ();
-      if (!error && plugin->exports_procedures)
-	{
-	  gpointer *sym_array;
-
-	  if (g_module_symbol (plugin->gmodule,
-			       BSE_EXPORT_SYMBOL (Procedure),
-			       (gpointer) &sym_array))
-	    {
-	      plugin->e_procs = sym_array;
-	      need_reinit |= TRUE;
-	    }
-	  else
-	    error = g_module_error ();
-	}
-      if (!error && plugin->exports_objects)
-	{
-	  gpointer *sym_array;
-
-	  if (g_module_symbol (plugin->gmodule,
-			       BSE_EXPORT_SYMBOL (Object),
-			       (gpointer) &sym_array))
-	    {
-	      plugin->e_objects = sym_array;
-	      need_reinit |= TRUE;
-	    }
-	  else
-	    error = g_module_error ();
-	}
       if (!error && plugin->exports_enums)
 	{
 	  gpointer *sym_array;
-
+	  
 	  if (g_module_symbol (plugin->gmodule,
 			       BSE_EXPORT_SYMBOL (Enum),
 			       (gpointer) &sym_array))
@@ -371,21 +442,69 @@ bse_plugin_ref (BsePlugin *plugin)
 	  else
 	    error = g_module_error ();
 	}
-
+      if (!error && plugin->exports_objects)
+	{
+	  gpointer *sym_array;
+	  
+	  if (g_module_symbol (plugin->gmodule,
+			       BSE_EXPORT_SYMBOL (Object),
+			       (gpointer) &sym_array))
+	    {
+	      plugin->e_objects = sym_array;
+	      need_reinit |= TRUE;
+	    }
+	  else
+	    error = g_module_error ();
+	}
+      if (!error && plugin->exports_procedures)
+	{
+	  gpointer *sym_array;
+	  
+	  if (g_module_symbol (plugin->gmodule,
+			       BSE_EXPORT_SYMBOL (Procedure),
+			       (gpointer) &sym_array))
+	    {
+	      plugin->e_procs = sym_array;
+	      need_reinit |= TRUE;
+	    }
+	  else
+	    error = g_module_error ();
+	}
+      if (!error && plugin->exports_file_handlers)
+	{
+	  gpointer *sym_array;
+	  
+	  if (g_module_symbol (plugin->gmodule,
+			       BSE_EXPORT_SYMBOL (FileHandler),
+			       (gpointer) &sym_array))
+	    {
+	      if (sym_array)
+		{
+		  plugin->e_file_handlers = *sym_array;
+		  need_reinit |= TRUE;
+		}
+	      else
+		error = "Failed to refetch file handler types";
+	    }
+	  else
+	    error = g_module_error ();
+	}
+      
       if (!error)
 	error = bse_plugin_reinit_type_ids (plugin);
-
+      
       if (error)
 	g_error ("Fatal plugin error, failed to reinitialize plugin: %s", error);
     }
-  plugin->module_refs++;
+  else
+    plugin->use_count++;
 }
 
 static void
 bse_plugin_unload (BsePlugin *plugin)
 {
   g_return_if_fail (plugin->gmodule != NULL && plugin->fname != NULL);
-
+  
   g_module_close (plugin->gmodule);
   plugin->gmodule = NULL;
   
@@ -393,37 +512,46 @@ bse_plugin_unload (BsePlugin *plugin)
   plugin->e_procs = NULL;
   plugin->e_objects = NULL;
   plugin->e_enums = NULL;
-
-  BSE_IF_DEBUG (PLUGINS)
-    g_message ("unloaded-plugin \"%s\"", plugin->name);
+  plugin->e_file_handlers = NULL;
+  
+  DEBUG ("unloaded-plugin \"%s\"", plugin->name);
 }
 
-void
-bse_plugin_unref (BsePlugin *plugin)
+static void
+bse_plugin_unuse (GTypePlugin *gplugin)
 {
-  g_return_if_fail (plugin != NULL);
-  g_return_if_fail (plugin->module_refs > 0);
+  BsePlugin *plugin = BSE_PLUGIN (gplugin);
   
-  plugin->module_refs--;
-  if (!plugin->module_refs)
-    /* FIXME: need timeout handler for module unloading */
-    bse_plugin_unload (plugin);
+  g_return_if_fail (plugin->use_count > 0);
+  
+  plugin->use_count--;
+  if (!plugin->use_count)
+    {
+      if (plugin->fname)
+	bse_plugin_unload (plugin);
+      else
+	{
+	  g_warning (G_STRLOC ": attempt to unload builtin plugin due to use_count==0");
+	  plugin->use_count = BUILTIN_USE_COUNT;
+	}
+    }
+  g_object_unref (G_OBJECT (plugin));
 }
 
 #define POINTER_ADD(p,n_bytes)	((void*) (((const char*) (p)) + (unsigned long) (n_bytes)))
 
 static inline const BseExportSpec*
 bse_plugin_get_export_spec (BsePlugin    *plugin,
-			    BseType       type,
+			    GType         type,
 			    gconstpointer export_specs,
 			    guint         spec_size)
 {
-  g_return_val_if_fail (export_specs && spec_size > sizeof (BseExportAny), NULL);
-
+  g_return_val_if_fail (export_specs && spec_size > sizeof (GType*), NULL);
+  
   if (export_specs)
     {
       const BseExportSpec* spec;
-
+      
       for (spec = export_specs; spec->type_p; spec = POINTER_ADD (spec, spec_size))
 	if (*(spec->type_p) == type)
 	  return spec;
@@ -432,53 +560,70 @@ bse_plugin_get_export_spec (BsePlugin    *plugin,
   return NULL;
 }
 
-void
-bse_plugin_complete_info (BsePlugin   *plugin,
-			  BseType      type,
-			  BseTypeInfo *type_info)
+static void
+enum_flags_complete_info (const BseExportSpec *spec,
+			  GTypeInfo           *info)
+{
+  const BseExportEnum *espec = &spec->s_enum;
+  GType type = *(espec->type_p);
+  
+  if (G_TYPE_IS_ENUM (type))
+    g_enum_complete_type_info (type, info, espec->values);
+  else if (G_TYPE_IS_FLAGS (type))
+    g_flags_complete_type_info (type, info, espec->values);
+  else
+    g_assert_not_reached ();
+}
+
+static void
+bse_plugin_complete_info (GTypePlugin     *gplugin,
+			  GType            type,
+			  GTypeInfo       *type_info,
+			  GTypeValueTable *value_vtable)
 {
   const BseExportSpec *export_spec = NULL;
   gconstpointer specs_p = NULL;
   guint spec_size = 0;
-  void (*complete_info) (const BseExportSpec *, BseTypeInfo *) = NULL;
+  void (*complete_info) (const BseExportSpec *, GTypeInfo *) = NULL;
+  BsePlugin *plugin = BSE_PLUGIN (gplugin);
   
   g_return_if_fail (plugin != NULL);
-  g_return_if_fail (plugin->module_refs > 0);
-
+  g_return_if_fail (plugin->use_count > 0);
+  
   /* we are an internal function, and thusly need to behave pretty conservative,
    * so in effect we only guarrantee to always fill the type_info structure we
    * get passed.
    */
   
-  switch (BSE_FUNDAMENTAL_TYPE (type))
+  switch (G_TYPE_FUNDAMENTAL (type))
     {
+    case G_TYPE_ENUM:
+    case G_TYPE_FLAGS:
+      specs_p = plugin->e_enums;
+      spec_size = sizeof (BseExportEnum);
+      complete_info = enum_flags_complete_info;
+      break;
+    case G_TYPE_OBJECT:
+      specs_p = plugin->e_objects;
+      spec_size = sizeof (BseExportObject);
+      complete_info = bse_object_complete_info;
+      break;
     case BSE_TYPE_PROCEDURE:
       specs_p = plugin->e_procs;
       spec_size = sizeof (BseExportProcedure);
       complete_info = bse_procedure_complete_info;
       break;
-    case BSE_TYPE_OBJECT:
-      specs_p = plugin->e_objects;
-      spec_size = sizeof (BseExportObject);
-      complete_info = bse_object_complete_info;
-      break;
-    case BSE_TYPE_ENUM:
-    case BSE_TYPE_FLAGS:
-      specs_p = plugin->e_enums;
-      spec_size = sizeof (BseExportEnum);
-      complete_info = bse_enum_complete_info;
-      break;
     default:
       g_assert_not_reached ();
       break;
     }
-
+  
   export_spec = bse_plugin_get_export_spec (plugin, type, specs_p, spec_size);
   if (!export_spec || !complete_info)
     g_error ("unable to find export spec for `%s' in \"%s\"",
-	     bse_type_name (type),
+	     g_type_name (type),
 	     plugin->name);
-
+  
   complete_info (export_spec, type_info);
 }
 
@@ -486,19 +631,19 @@ static inline BsePlugin*
 bse_plugin_find (GModule *gmodule)
 {
   GSList *slist;
-
+  
   for (slist = bse_plugins; slist; slist = slist->next)
     {
       BsePlugin *plugin = slist->data;
-
+      
       if (plugin->gmodule == gmodule)
 	return plugin;
     }
-
+  
   return NULL;
 }
 
-gchar*
+const gchar*
 bse_plugin_check_load (const gchar *_file_name)
 {
   const gint TOKEN_DLNAME = G_TOKEN_LAST + 1;
@@ -513,7 +658,7 @@ bse_plugin_check_load (const gchar *_file_name)
   gchar *error = NULL;
   
   g_return_val_if_fail (file_name != NULL, NULL);
-
+  
   /* open libtool archive
    */
   fd = open (file_name, O_RDONLY, 0);
@@ -521,7 +666,7 @@ bse_plugin_check_load (const gchar *_file_name)
     return (errno == ENOENT || errno == ENOTDIR || errno == ELOOP ?
 	    bse_error_blurb (BSE_ERROR_FILE_NOT_FOUND) :
 	    "Unable to access plugin");
-
+  
   /* and search libtool's dlname specification
    */
   scanner = g_scanner_new (NULL);
@@ -543,18 +688,18 @@ bse_plugin_check_load (const gchar *_file_name)
       g_scanner_destroy (scanner);
       close (fd);
       
-      return "Plugin broken";
+      return "Plugin's dlname broken";
     }
-
+  
   /* construct real module name
    */
-  string = g_dirname (file_name);
+  string = g_path_get_dirname (file_name);
   file_name = g_strconcat (string, G_DIR_SEPARATOR_S, scanner->value.v_string, NULL);
   g_free (string);
   
   g_scanner_destroy (scanner);
   close (fd);
-
+  
   /* load module
    */
   gmodule = g_module_open (file_name, 0);
@@ -562,18 +707,18 @@ bse_plugin_check_load (const gchar *_file_name)
   if (!gmodule)
     {
       g_free (file_name);
-
+      
       return g_module_error ();
     }
-
+  
   if (bse_plugin_find (gmodule))
     {
       g_module_close (gmodule);
       g_free (file_name);
-
+      
       return "Plugin already loaded";
     }
-
+  
   /* check whether this is a BSE module
    */
   if (!g_module_symbol (gmodule, BSE_EXPORT_SYMBOL (Begin), (gpointer) &sym_begin) ||
@@ -585,37 +730,39 @@ bse_plugin_check_load (const gchar *_file_name)
       
       return "Not a BSE Plugin";
     }
-
+  
   if (bse_plugin_lookup (*sym_begin))
     {
       g_module_close (gmodule);
       g_free (file_name);
-
+      
       return "Plugin already registered (clone?)";
     }
-
+  
   /* create plugin and feature type registration
    */
-  plugin = g_new0 (BsePlugin, 1); /* FIXME: alloc_only */
+  plugin = g_object_new (BSE_TYPE_PLUGIN, NULL);
+  g_free (plugin->name);
   plugin->name = g_strdup (*sym_begin);
   plugin->fname = file_name;
   plugin->gmodule = gmodule;
-  plugin->module_refs = 0;
-
-  if (g_module_symbol (gmodule, BSE_EXPORT_SYMBOL (Procedure), (gpointer) &sym_array))
-    error = (gchar*) bse_plugin_register_types (plugin, sym_array, BSE_EXPORT_TYPE_PROCS);
-  if (!error && g_module_symbol (gmodule, BSE_EXPORT_SYMBOL (Object), (gpointer) &sym_array))
-    error = (gchar*) bse_plugin_register_types (plugin, sym_array, BSE_EXPORT_TYPE_OBJECTS);
+  
   if (!error && g_module_symbol (gmodule, BSE_EXPORT_SYMBOL (Enum), (gpointer) &sym_array))
     {
       if (sym_array)
-	error = (gchar*) bse_plugin_register_types (plugin, *sym_array, BSE_EXPORT_TYPE_ENUMS);
+	error = (gchar*) bse_plugin_register_exports (plugin, *sym_array, BSE_EXPORT_TYPE_ENUMS);
       else
 	error = "Failed to fetch enum types";
     }
-
+  if (!error && g_module_symbol (gmodule, BSE_EXPORT_SYMBOL (Object), (gpointer) &sym_array))
+    error = (gchar*) bse_plugin_register_exports (plugin, sym_array, BSE_EXPORT_TYPE_OBJECTS);
+  if (g_module_symbol (gmodule, BSE_EXPORT_SYMBOL (Procedure), (gpointer) &sym_array))
+    error = (gchar*) bse_plugin_register_exports (plugin, sym_array, BSE_EXPORT_TYPE_PROCS);
+  if (!error && g_module_symbol (gmodule, BSE_EXPORT_SYMBOL (FileHandler), (gpointer) &sym_array))
+    error = (gchar*) bse_plugin_register_exports (plugin, sym_array, BSE_EXPORT_TYPE_FILE_HANDLERS);
+  
   bse_plugins = g_slist_prepend (bse_plugins, plugin);
-
+  
   bse_plugin_unload (plugin);
   
   return error;
@@ -625,17 +772,17 @@ BsePlugin*
 bse_plugin_lookup (const gchar *name)
 {
   GSList *slist;
-
+  
   g_return_val_if_fail (name != NULL, NULL);
-
+  
   for (slist = bse_plugins; slist; slist = slist->next)
     {
       BsePlugin *plugin = slist->data;
-
+      
       if (bse_string_equals (name, plugin->name))
 	return plugin;
     }
-
+  
   return NULL;
 }
 
@@ -643,49 +790,10 @@ bse_plugin_lookup (const gchar *name)
 #include <sys/types.h>
 #include <dirent.h>
 
-GList*
-bse_plugin_dir_list_files (const gchar *_dir_list)
+GSList*
+bse_plugin_dir_list_files (const gchar *dir_list)
 {
-  GList *file_list = NULL;
-  gchar *free_me, *dir_list = g_strdup (_dir_list);
-
-  g_return_val_if_fail (dir_list != NULL, NULL);
-
-  free_me = dir_list;
-
-  while (dir_list && dir_list[0])
-    {
-      gchar *name = dir_list;
-      DIR *dd;
-
-      dir_list = strchr (dir_list, G_SEARCHPATH_SEPARATOR);
-      if (dir_list)
-	*(dir_list++) = 0;
-
-      dd = opendir (name);
-      if (dd)
-	{
-	  struct dirent *d_entry = readdir (dd);
-
-	  while (d_entry)
-	    {
-	      guint l = strlen (d_entry->d_name);
-
-	      /* select libtool archives */
-	      if (l > 3 &&
-		  d_entry->d_name[l - 3] == '.' &&
-		  d_entry->d_name[l - 2] == 'l' &&
-		  d_entry->d_name[l - 1] == 'a')
-		file_list = g_list_prepend (file_list, g_strconcat (name,
-								    "/",
-								    d_entry->d_name,
-								    NULL));
-	      d_entry = readdir (dd);
-	    }
-	  closedir (dd);
-	}
-    }
-  g_free (free_me);
-
-  return g_list_sort (file_list, (GCompareFunc) strcmp);
+  GSList *slist = bse_search_path_list_files (dir_list, "*.la", NULL, 0);
+  
+  return g_slist_sort (slist, (GCompareFunc) strcmp);
 }
