@@ -15,10 +15,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  */
-#include        "bse.h"
-#include        "../PKG_config.h"
+#include        "bsecategories.h"
+#include        "bsesource.h"
+#include        "bseprocedure.h"
+#include        "bsemain.h"
+#include        "bsestandardsynths.h"
+#include        "PKG_config.h"
 #include	<stdio.h>
 #include	<stdlib.h>
+#include	<string.h>
 #include	<unistd.h>
 #include	<sys/stat.h>
 #include	<fcntl.h>
@@ -117,34 +122,31 @@ show_nodes (GType        type,
 static void
 show_cats (void)
 {
-  BseCategory *cats;
-  guint i, n_cats;
+  BseCategorySeq *cseq;
+  guint i;
 
-  cats = bse_categories_match ("*", &n_cats);
-  for (i = 0; i < n_cats; i++)
+  cseq = bse_categories_match ("*");
+  for (i = 0; i < cseq->n_cats; i++)
     fprintf (f_out, "%s\t(%s)\n",
-	     cats[i].category,
-	     g_type_name (cats[i].type));
-  g_free (cats);
+	     cseq->cats[i]->category,
+	     cseq->cats[i]->type);
+  bse_category_seq_free (cseq);
 }
 
 static void
 show_procdoc (void)
 {
-  BseCategory *cats;
-  guint i, n_cats;
+  BseCategorySeq *cseq;
+  guint i;
   const gchar *nullstr = ""; // "???";
 
-  cats = bse_categories_match_typed ("*", BSE_TYPE_PROCEDURE, &n_cats);
-  for (i = 0; i < n_cats; i++)
+  cseq = bse_categories_match_typed ("*", BSE_TYPE_PROCEDURE);
+  for (i = 0; i < cseq->n_cats; i++)
     {
-      BseProcedureClass *class = g_type_class_ref (cats[i].type);
-      gchar *pname = g_type_name_to_sname (g_type_name (cats[i].type));
+      BseProcedureClass *class = g_type_class_ref (g_type_from_name (cseq->cats[i]->type));
+      gchar *pname = g_type_name_to_sname (cseq->cats[i]->type);
       guint j;
 
-      if (pname[0] == 'b' && pname[1] == 's' && pname[2] == 'e')
-	pname[2] = 'w';
-
       fprintf (f_out, "/**\n * %s\n", pname);
       for (j = 0; j < class->n_in_pspecs; j++)
 	{
@@ -170,7 +172,7 @@ show_procdoc (void)
       g_type_class_unref (class);
       g_free (pname);
     }
-  g_free (cats);
+  bse_category_seq_free (cseq);
 }
 
 static gint
@@ -187,10 +189,12 @@ help (gchar *arg)
   fprintf (stderr, "       -i       specifiy incremental indent string\n");
   fprintf (stderr, "       -s       specifiy line spacing\n");
   fprintf (stderr, "qualifiers:\n");
-  fprintf (stderr, "       froots   iterate over fundamental roots\n");
-  fprintf (stderr, "       tree     print BSE type tree\n");
-  fprintf (stderr, "       cats     print categories\n");
-  fprintf (stderr, "       procdoc  print procedure documentation\n");
+  fprintf (stderr, "       froots     iterate over fundamental roots\n");
+  fprintf (stderr, "       tree       print BSE type tree\n");
+  fprintf (stderr, "       cats       print categories\n");
+  fprintf (stderr, "       procdoc    print procedure documentation\n");
+  fprintf (stderr, "       synthlist  list standard synths\n");
+  fprintf (stderr, "       synth <x>  dump standard synth <x> definition\n");
 
   return arg != NULL;
 }
@@ -203,6 +207,8 @@ main (gint   argc,
   gboolean gen_tree = 0;
   gboolean gen_cats = 0;
   gboolean gen_procdoc = 0;
+  gboolean list_synths = 0;
+  gchar *show_synth = NULL;
   guint i;
   gchar *iindent = "";
   
@@ -210,7 +216,7 @@ main (gint   argc,
 
   g_thread_init (NULL);
 
-  bse_init (&argc, &argv, NULL);
+  bse_init_intern (&argc, &argv, NULL);
   
   root = BSE_TYPE_OBJECT;
 
@@ -271,6 +277,14 @@ main (gint   argc,
 	{
 	  gen_froots = 1;
 	}
+      else if (strcmp ("synthlist", argv[i]) == 0)
+	{
+	  list_synths = 1;
+	}
+      else if (strcmp ("synth", argv[i]) == 0 && i + 1 < argc)
+	{
+	  show_synth = argv[++i];
+	}
       else if (strcmp ("tree", argv[i]) == 0)
 	{
 	  gen_tree = 1;
@@ -285,7 +299,7 @@ main (gint   argc,
 	}
       else if (strcmp ("-p", argv[i]) == 0)
 	{
-	  bsw_register_plugins (NULL, TRUE, NULL, NULL, NULL);
+	  // FIXME: bsw_register_plugins (NULL, TRUE, NULL, NULL, NULL);
 	}
       else if (strcmp ("-h", argv[i]) == 0)
 	{
@@ -298,7 +312,7 @@ main (gint   argc,
       else
 	return help (argv[i]);
     }
-  if (!gen_froots && !gen_tree && !gen_cats && !gen_procdoc)
+  if (!gen_froots && !gen_tree && !gen_cats && !gen_procdoc && !list_synths && !show_synth)
     return help (argv[i-1]);
 
   if (!indent_inc)
@@ -315,7 +329,7 @@ main (gint   argc,
   if (gen_froots)
     {
       root = ~0;
-      for (i = 0; i < 256; i++)
+      for (i = 0; i <= G_TYPE_FUNDAMENTAL_MAX; i += G_TYPE_MAKE_FUNDAMENTAL (1))
 	{
 	  gchar *name = g_type_name (i);
 	  
@@ -327,6 +341,18 @@ main (gint   argc,
     show_cats ();
   if (gen_procdoc)
     show_procdoc ();
+  if (list_synths)
+    {
+      GSList *node = bse_standard_synth_get_list ();
+      for (; node; node = node->next)
+	g_print ("  %s\n", (gchar*) node->data);
+    }
+  if (show_synth)
+    {
+      gchar *text = bse_standard_synth_lookup (show_synth, NULL);
+      g_print ("%s", text);
+      g_free (text);
+    }
   
   return 0;
 }
