@@ -23,11 +23,15 @@
 #include "gslopschedule.h"
 #include "gslieee754.h"
 #include <string.h>
+#include <unistd.h>
 #include <sys/poll.h>
 #include <sys/time.h>
 #include <errno.h>
 
 
+#define DEBUG        sfi_debug_keyfunc ("engine")
+#define JOB_DEBUG    sfi_debug_keyfunc ("job")
+#define FJOB_DEBUG   sfi_debug_keyfunc ("fjob")
 
 #define	NODE_FLAG_RECONNECT(node)  G_STMT_START { /*(node)->needs_reset = (node)->module.klass->reset != NULL*/; } G_STMT_END
 
@@ -59,6 +63,14 @@ struct _Poll
   GPollFD    *fds;
   GslFreeFunc free_func;
 };
+typedef struct _Timer Timer;
+struct _Timer
+{
+  Timer             *next;
+  GslEngineTimerFunc timer_func;
+  gpointer           data;
+  GslFreeFunc        free_func;
+};
 
 
 /* --- prototypes --- */
@@ -70,6 +82,7 @@ static gboolean	       master_need_reflow = FALSE;
 static gboolean	       master_need_process = FALSE;
 static EngineNode     *master_consumer_list = NULL;
 const gfloat           gsl_engine_master_zero_block[GSL_STREAM_MAX_VALUES] = { 0, }; /* FIXME */
+static Timer	      *master_timer_list = NULL;
 static Poll	      *master_poll_list = NULL;
 static guint           master_n_pollfds = 0;
 static guint           master_pollfds_changed = FALSE;
@@ -138,7 +151,7 @@ master_idisconnect_node (EngineNode *node,
   was_consumer = ENGINE_NODE_IS_CONSUMER (src_node);
   src_node->outputs[ostream].n_outputs -= 1;
   src_node->module.ostreams[ostream].connected = 0; /* scheduler update */
-  src_node->output_nodes = gsl_ring_remove (src_node->output_nodes, node);
+  src_node->output_nodes = sfi_ring_remove (src_node->output_nodes, node);
   NODE_FLAG_RECONNECT (node);
   NODE_FLAG_RECONNECT (src_node);
   /* update suspension state of input */
@@ -167,7 +180,7 @@ master_jdisconnect_node (EngineNode *node,
   was_consumer = ENGINE_NODE_IS_CONSUMER (src_node);
   src_node->outputs[ostream].n_outputs -= 1;
   src_node->module.ostreams[ostream].connected = 0; /* scheduler update */
-  src_node->output_nodes = gsl_ring_remove (src_node->output_nodes, node);
+  src_node->output_nodes = sfi_ring_remove (src_node->output_nodes, node);
   NODE_FLAG_RECONNECT (node);
   NODE_FLAG_RECONNECT (src_node);
   /* update suspension state of input */
@@ -199,6 +212,7 @@ master_process_job (GslJob *job)
     {
       EngineNode *node, *src_node;
       Poll *poll, *poll_last;
+      Timer *timer;
       guint istream, jstream, ostream, con;
       EngineFlowJob *fjob;
       gboolean was_consumer;
@@ -312,7 +326,7 @@ master_process_job (GslJob *job)
       was_consumer = ENGINE_NODE_IS_CONSUMER (src_node);
       src_node->outputs[ostream].n_outputs += 1;
       src_node->module.ostreams[ostream].connected = 0; /* scheduler update */
-      src_node->output_nodes = gsl_ring_append (src_node->output_nodes, node);
+      src_node->output_nodes = sfi_ring_append (src_node->output_nodes, node);
       NODE_FLAG_RECONNECT (node);
       NODE_FLAG_RECONNECT (src_node);
       /* update suspension state of input */
@@ -339,7 +353,7 @@ master_process_job (GslJob *job)
       was_consumer = ENGINE_NODE_IS_CONSUMER (src_node);
       src_node->outputs[ostream].n_outputs += 1;
       src_node->module.ostreams[ostream].connected = 0; /* scheduler update */
-      src_node->output_nodes = gsl_ring_append (src_node->output_nodes, node);
+      src_node->output_nodes = sfi_ring_append (src_node->output_nodes, node);
       NODE_FLAG_RECONNECT (node);
       NODE_FLAG_RECONNECT (src_node);
       /* update suspension state of input */
@@ -400,7 +414,7 @@ master_process_job (GslJob *job)
       if (job->data.poll.n_fds + master_n_pollfds > GSL_ENGINE_MAX_POLLFDS)
 	g_error ("adding poll job exceeds maximum number of poll-fds (%u > %u)",
 		 job->data.poll.n_fds + master_n_pollfds, GSL_ENGINE_MAX_POLLFDS);
-      poll = gsl_new_struct0 (Poll, 1);
+      poll = sfi_new_struct0 (Poll, 1);
       poll->poll_func = job->data.poll.poll_func;
       poll->data = job->data.poll.data;
       poll->free_func = job->data.poll.free_func;
@@ -440,12 +454,22 @@ master_process_job (GslJob *job)
 	      master_n_pollfds -= poll_last->n_fds;
 	      master_pollfds_changed = TRUE;
 	    }
-	  gsl_delete_struct (Poll, poll_last);
+	  sfi_delete_struct (Poll, poll_last);
 	}
       else
 	g_warning (G_STRLOC ": failed to remove unknown poll function %p(%p)",
 		   job->data.poll.poll_func, job->data.poll.data);
       break;
+    case ENGINE_JOB_ADD_TIMER:
+      JOB_DEBUG ("add timer %p(%p)", job->data.timer.timer_func, job->data.timer.data);
+      timer = sfi_new_struct0 (Timer, 1);
+      timer->timer_func = job->data.timer.timer_func;
+      timer->data = job->data.timer.data;
+      timer->free_func = job->data.timer.free_func;
+      job->data.timer.free_func = NULL;		/* don't free data this round */
+      timer->next = master_timer_list;
+      master_timer_list = timer;
+      break;
     default:
       g_assert_not_reached ();
     }
@@ -482,6 +506,35 @@ master_poll_check (glong   *timeout_p,
   master_need_process = need_processing;
 }
 
+static void
+master_tick_stamp_inc (void)
+{
+  Timer *timer, *last = NULL;
+  guint64 new_stamp;
+  _gsl_tick_stamp_inc ();
+  new_stamp = GSL_TICK_STAMP;
+  timer = master_timer_list;
+  while (timer)
+    {
+      Timer *next = timer->next;
+      if (!timer->timer_func (timer->data, new_stamp))
+	{
+	  GslTrans *trans = gsl_trans_open ();
+	  if (last)
+	    last->next = next;
+	  else
+	    master_timer_list = next;
+	  /* free timer data in user thread */
+	  gsl_trans_add (trans, gsl_job_add_timer (timer->timer_func, timer->data, timer->free_func));
+	  gsl_trans_dismiss (trans);
+	  sfi_delete_struct (Timer, timer);
+	}
+      else
+	last = timer;
+      timer = next;
+    }
+}
+
 static void
 propagate_resume (EngineNode *node)
 {
@@ -643,7 +696,7 @@ master_process_flow (void)
   
   g_assert (gsl_fpu_okround () == TRUE);
   
-  MAS_DEBUG ("process_flow");
+  DEBUG ("process_flow");
   if (master_schedule)
     {
       EngineNode *node;
@@ -706,7 +759,7 @@ master_process_flow (void)
       _engine_wait_on_unprocessed ();
       
       _engine_unset_schedule (master_schedule);
-      _gsl_tick_stamp_inc ();
+      master_tick_stamp_inc ();
       _engine_recycle_const_values ();
     }
   master_need_process = FALSE;
@@ -719,7 +772,7 @@ master_reschedule_flow (void)
   
   g_return_if_fail (master_need_reflow == TRUE);
   
-  MAS_DEBUG ("flow_reschedule");
+  DEBUG ("flow_reschedule");
   if (!master_schedule)
     master_schedule = _engine_schedule_new ();
   else
@@ -780,9 +833,9 @@ _engine_master_prepare (GslEngineLoop *loop)
   if (need_dispatch)
     loop->timeout = 0;
   
-  MAS_DEBUG ("PREPARE: need_dispatch=%u timeout=%6ld n_fds=%u",
-	     need_dispatch,
-	     loop->timeout, loop->n_fds);
+  DEBUG ("PREPARE: need_dispatch=%u timeout=%6ld n_fds=%u",
+	 need_dispatch,
+	 loop->timeout, loop->n_fds);
   
   return need_dispatch;
 }
@@ -812,7 +865,7 @@ _engine_master_check (const GslEngineLoop *loop)
       need_dispatch = master_need_process;
     }
   
-  MAS_DEBUG ("CHECK: need_dispatch=%u", need_dispatch);
+  DEBUG ("CHECK: need_dispatch=%u", need_dispatch);
   
   return need_dispatch;
 }
@@ -851,25 +904,26 @@ _engine_master_dispatch (void)
 }
 
 void
-_engine_master_thread (gpointer data)
+_engine_master_thread (EngineMasterData *mdata)
 {
   gboolean run = TRUE;
-  
+
   /* assert sane configuration checks, since we're simply casting structures */
   g_assert (sizeof (struct pollfd) == sizeof (GPollFD) &&
 	    G_STRUCT_OFFSET (GPollFD, fd) == G_STRUCT_OFFSET (struct pollfd, fd) &&
 	    G_STRUCT_OFFSET (GPollFD, events) == G_STRUCT_OFFSET (struct pollfd, events) &&
 	    G_STRUCT_OFFSET (GPollFD, revents) == G_STRUCT_OFFSET (struct pollfd, revents));
   
-  /* add the thread wakeup pipe to master pollfds, so we get woken
-   * up in time (even though we evaluate the pipe contents later)
+  /* add the thread wakeup pipe to master pollfds,
+   * so we get woken  up in time.
    */
-  gsl_thread_get_pollfd (master_pollfds);
-  master_n_pollfds += 1;
+  master_pollfds[0].fd = mdata->wakeup_pipe[0];
+  master_pollfds[0].events = G_IO_IN;
+  master_n_pollfds = 1;
   master_pollfds_changed = TRUE;
-  
+
   toyprof_stampinit ();
-  
+
   while (run)
     {
       GslEngineLoop loop;
@@ -895,8 +949,19 @@ _engine_master_thread (gpointer data)
       if (need_dispatch)
 	_engine_master_dispatch ();
       
-      /* handle thread pollfd messages */
-      run = gsl_thread_sleep (0);
+      /* clear wakeup pipe */
+      {
+	guint8 data[64];
+	gint l;
+	do
+	  l = read (mdata->wakeup_pipe[0], data, sizeof (data));
+	while ((l < 0 && errno == EINTR) || l == sizeof (data));
+      }
+
+      /* wakeup user thread if necessary */
+      if (gsl_engine_has_garbage ())
+	sfi_thread_wakeup (mdata->user_thread);
     }
 }
+
 /* vim:set ts=8 sts=2 sw=2: */
