@@ -18,8 +18,8 @@
  */
 #include "bsesequencer.h"
 
-#include <gsl/gslengine.h>
-#include <gsl/gslieee754.h>
+#include <bse/gslengine.h>
+#include <bse/gslieee754.h>
 
 
 enum {
@@ -90,27 +90,24 @@ bse_sequencer_class_init (BseSequencerClass *class)
   
   bse_object_class_add_param (object_class, "Sequence",
 			      PARAM_LENGTH,
-			      bse_param_spec_uint ("length", "Length", NULL,
-						   1, 128,
-						   8, 4,
-						   BSE_PARAM_GUI | BSE_PARAM_HINT_SCALE));
+			      sfi_pspec_int ("length", "Length", NULL,
+					     8, 1, 128, 4,
+					     SFI_PARAM_GUI SFI_PARAM_HINT_SCALE));
   bse_object_class_add_param (object_class, "Sequence",
 			      PARAM_NOTES,
-			      g_param_spec_boxed ("notes", "Notes", NULL,
-						  BSW_TYPE_NOTE_SEQUENCE,
-						  BSE_PARAM_DEFAULT));
+			      bse_param_spec_boxed ("notes", "Notes", NULL,
+						    BSE_TYPE_NOTE_SEQUENCE,
+						    "note-sequence:" SFI_PARAM_DEFAULT));
   bse_object_class_add_param (object_class, "Sequence",
 			      PARAM_TRANSPOSE,
-			      bse_param_spec_int ("transpose", "Transpose", NULL,
-						  -36, +36,
-						  0, 3,
-						  BSE_PARAM_DEFAULT | BSE_PARAM_HINT_SCALE));
+			      sfi_pspec_int ("transpose", "Transpose", NULL,
+					     0, -36, +36, 3,
+					     SFI_PARAM_DEFAULT SFI_PARAM_HINT_SCALE));
   bse_object_class_add_param (object_class, "Sequence",
 			      PARAM_COUNTER,
-			      bse_param_spec_float ("counter", "Timing [ms]", NULL,
-						    0, 1000,
-						    100, 5,
-						    BSE_PARAM_DEFAULT | BSE_PARAM_HINT_SCALE));
+			      sfi_pspec_real ("counter", "Timing [ms]", NULL,
+					      100, 0, 1000, 5,
+					      SFI_PARAM_DEFAULT SFI_PARAM_HINT_SCALE SFI_PARAM_FLOAT));
   
   ochannel = bse_source_class_add_ochannel (source_class, "Freq Out", "Frequency Signal");
   g_assert (ochannel == BSE_SEQUENCER_OCHANNEL_FREQ);
@@ -121,7 +118,9 @@ bse_sequencer_class_init (BseSequencerClass *class)
 static void
 bse_sequencer_init (BseSequencer *seq)
 {
-  seq->sdata = bsw_note_sequence_new (8);
+  seq->sdata = bse_note_sequence_new ();
+  bse_note_sequence_resize (seq->sdata, 8);
+  seq->sdata->offset = SFI_NOTE_C (SFI_KAMMER_OCTAVE);
   seq->n_freq_values = 0;
   seq->freq_values = NULL;
   seq->transpose = 0;
@@ -131,9 +130,9 @@ static void
 bse_sequencer_finalize (GObject *object)
 {
   BseSequencer *seq = BSE_SEQUENCER (object);
-
-  bsw_note_sequence_free (seq->sdata);
-
+  
+  bse_note_sequence_free (seq->sdata);
+  
   /* chain parent class' handler */
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -146,28 +145,46 @@ bse_sequencer_set_property (BseSequencer *seq,
 {
   switch (param_id)
     {
-      BswNoteSequence *sdata;
+      BseNoteSequence *sdata;
     case PARAM_LENGTH:
-      if (g_value_get_uint (value) != seq->sdata->n_notes)
+      if (sfi_value_get_int (value) != bse_note_sequence_length (seq->sdata))
 	{
-	  seq->sdata = bsw_note_sequence_resize (seq->sdata, g_value_get_uint (value));
+	  bse_note_sequence_resize (seq->sdata, sfi_value_get_int (value));
 	  bse_sequencer_update_modules (seq);
 	  g_object_notify (seq, "notes");
 	}
       break;
     case PARAM_NOTES:
-      bsw_note_sequence_free (seq->sdata);
-      sdata = g_value_get_boxed (value);
-      seq->sdata = sdata ? bsw_note_sequence_copy (sdata) : bsw_note_sequence_new (8);
+      bse_note_sequence_free (seq->sdata);
+      sdata = bse_value_get_boxed (value);
+      if (sdata)
+	{
+	  guint i, l, mnote = SFI_MAX_NOTE;
+	  seq->sdata = bse_note_sequence_copy_shallow (sdata);
+	  /* fixup offset */
+	  l = bse_note_sequence_length (seq->sdata);
+	  for (i = 0; i < l; i++)
+	    mnote = MIN (mnote, seq->sdata->notes->notes[i]);
+	  if (l && ABS (mnote - seq->sdata->offset) >= 12)
+	    seq->sdata->offset = (mnote < SFI_NOTE_A (SFI_NOTE_OCTAVE (mnote)) ?
+				  SFI_NOTE_C (SFI_NOTE_OCTAVE (mnote)) :
+				  SFI_NOTE_A (SFI_NOTE_OCTAVE (mnote)));
+	}
+      else
+	{
+	  seq->sdata = bse_note_sequence_new ();
+	  bse_note_sequence_resize (seq->sdata, 8);
+	  seq->sdata->offset = SFI_NOTE_C (SFI_KAMMER_OCTAVE);
+	}
       bse_sequencer_update_modules (seq);
       g_object_notify (seq, "length");
       break;
     case PARAM_COUNTER:
-      seq->counter = g_value_get_float (value);
+      seq->counter = sfi_value_get_real (value);
       bse_sequencer_update_modules (seq);
       break;
     case PARAM_TRANSPOSE:
-      seq->transpose = g_value_get_int (value);
+      seq->transpose = sfi_value_get_int (value);
       bse_sequencer_update_modules (seq);
       break;
     default:
@@ -185,16 +202,16 @@ bse_sequencer_get_property (BseSequencer *seq,
   switch (param_id)
     {
     case PARAM_NOTES:
-      g_value_set_boxed (value, seq->sdata);
+      bse_value_set_boxed (value, seq->sdata);
       break;
     case PARAM_LENGTH:
-      g_value_set_uint (value, seq->sdata->n_notes);
+      sfi_value_set_int (value, bse_note_sequence_length (seq->sdata));
       break;
     case PARAM_COUNTER:
-      g_value_set_float (value, seq->counter);
+      sfi_value_set_real (value, seq->counter);
       break;
     case PARAM_TRANSPOSE:
-      g_value_set_int (value, seq->transpose);
+      sfi_value_set_int (value, seq->transpose);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (seq, param_id, pspec);
@@ -203,22 +220,22 @@ bse_sequencer_get_property (BseSequencer *seq,
 }
 
 static gfloat*
-freq_values_from_seq (BswNoteSequence *sdata,
+freq_values_from_seq (BseNoteSequence *sdata,
 		      gint             transpose)
 {
-  gfloat *v = g_new (gfloat, sdata->n_notes);
+  gfloat *v = g_new (gfloat, bse_note_sequence_length (sdata));
   guint i;
-
-  for (i = 0; i < sdata->n_notes; i++)
+  
+  for (i = 0; i < bse_note_sequence_length (sdata); i++)
     {
-      gint note = sdata->notes[i].note;
-
-      if (note == BSE_NOTE_VOID)
+      gint note = sdata->notes->notes[i];
+      
+      if (note == SFI_NOTE_VOID)
 	v[i] = 0;
       else
-	v[i] = BSE_VALUE_FROM_FREQ (bse_note_to_freq (CLAMP (note + transpose, BSE_MIN_NOTE, BSE_MAX_NOTE)));
+	v[i] = BSE_VALUE_FROM_FREQ (bse_note_to_freq (CLAMP (note + transpose, SFI_MIN_NOTE, SFI_MAX_NOTE)));
     }
-
+  
   return v;
 }
 
@@ -242,7 +259,7 @@ seq_access (GslModule *module,
 {
   SeqModule *smod = module->user_data;
   AccessData *d = data;
-
+  
   smod->n_values = d->n_values;
   smod->values = d->new_values;
   smod->counter = d->counter;
@@ -256,7 +273,7 @@ static void
 seq_access_free (gpointer data)
 {
   AccessData *d = data;
-
+  
   g_free (d->old_values);
   g_free (d);
 }
@@ -267,17 +284,17 @@ bse_sequencer_update_modules (BseSequencer *seq)
   if (BSE_SOURCE_PREPARED (seq))
     {
       AccessData *d = g_new (AccessData, 1);
-
+      
       d->old_values = seq->freq_values;
-
-      seq->n_freq_values = seq->sdata->n_notes;
+      
+      seq->n_freq_values = bse_note_sequence_length (seq->sdata);
       seq->freq_values = freq_values_from_seq (seq->sdata, seq->transpose);
-
+      
       d->n_values = seq->n_freq_values;
       d->new_values = seq->freq_values;
       d->counter = seq->counter / 1000.0 * gsl_engine_sample_freq ();
       d->counter = MAX (d->counter, 1);
-
+      
       bse_source_access_modules (BSE_SOURCE (seq),
 				 seq_access, d, seq_access_free,
 				 NULL);
@@ -292,11 +309,11 @@ sequencer_process (GslModule *module,
   gfloat *freq_out = GSL_MODULE_OBUFFER (module, BSE_SEQUENCER_OCHANNEL_FREQ);
   gfloat *nsync_out = GSL_MODULE_OBUFFER (module, BSE_SEQUENCER_OCHANNEL_NOTE_SYNC);
   gfloat *bound = freq_out + n_values;
-
+  
   while (freq_out < bound)
     {
       gfloat nval = smod->values[smod->index];
-
+      
       if (smod->c == 0)
 	{
 	  smod->c = smod->counter;
@@ -317,10 +334,10 @@ static void
 bse_sequencer_prepare (BseSource *source)
 {
   BseSequencer *seq = BSE_SEQUENCER (source);
-
-  seq->n_freq_values = seq->sdata->n_notes;
+  
+  seq->n_freq_values = bse_note_sequence_length (seq->sdata);
   seq->freq_values = freq_values_from_seq (seq->sdata, seq->transpose);
-
+  
   /* chain parent class' handler */
   BSE_SOURCE_CLASS (parent_class)->prepare (source);
 }
@@ -343,7 +360,7 @@ bse_sequencer_context_create (BseSource *source,
   BseSequencer *seq = BSE_SEQUENCER (source);
   SeqModule *smod = g_new0 (SeqModule, 1);
   GslModule *module;
-
+  
   smod->n_values = seq->n_freq_values;
   smod->values = seq->freq_values;
   smod->counter = seq->counter / 1000.0 * gsl_engine_sample_freq ();
@@ -367,11 +384,11 @@ static void
 bse_sequencer_reset (BseSource *source)
 {
   BseSequencer *seq = BSE_SEQUENCER (source);
-
+  
   g_free (seq->freq_values);
   seq->freq_values = NULL;
   seq->n_freq_values = 0;
-
+  
   /* chain parent class' handler */
   BSE_SOURCE_CLASS (parent_class)->reset (source);
 }
